#include "Node.h"

void CTree::Init(string pair, vector<string> &currencyPairs, string currency)
{
	Root = make_shared<CNode>();

	Root->Pair = pair;
	m_Currency = currency;
	FillTree(currencyPairs, Root, 0);
}

void CTree::FillTree(vector<string>& pairs, shared_ptr<CNode> node, int depth)
{
	if (depth < MAX_DEPTH - 1)
	{
		depth++;
		if (CurrencyOnLeft(node->Pair, m_Currency))
		{
			return;
		}
		else if (CurrencyOnRight(node->Pair, m_Currency))
		{
			return;
		}
		for (vector<string>::iterator i = pairs.begin(); i != pairs.end(); i++)
		{
			if (CommonOnLeft(node->Pair, *i))
			{
				shared_ptr<CNode> n = make_shared<CNode>();

				n->Pair = (*i);
				node->LeftPairs.insert(node->LeftPairs.end(), n);
			}
			if (CommonOnRight(node->Pair, *i))
			{
				shared_ptr<CNode> n = make_shared<CNode>();

				n->Pair = (*i);
				node->RightPairs.insert(node->RightPairs.end(), n);
			}
		}
		for (vector<shared_ptr<CNode>>::iterator i = node->LeftPairs.begin(); i != node->LeftPairs.end(); i++)
		{
			FillTree(pairs, (*i), depth);
		}
		for (vector<shared_ptr<CNode>>::iterator i = node->RightPairs.begin(); i != node->RightPairs.end(); i++)
		{
			FillTree(pairs, (*i), depth);
		}
	}
}

bool CTree::CurrencyOnLeft(string pair, string currency)
{
	if (LeftCurrency(pair) == currency)
	{
		return true;
	}
	return false;
}

bool CTree::CurrencyOnRight(string pair, string currency)
{
	if (RightCurrency(pair) == currency)
	{
		return true;
	}
	return false;
}

bool CTree::CommonOnLeft(string pair, string secondPair)
{
	if (LeftCurrency(pair) == RightCurrency(secondPair) || LeftCurrency(pair) == LeftCurrency(secondPair))
	{
		return true;
	}
	return false;
}

bool CTree::CommonOnRight(string pair, string secondPair)
{
	if (RightCurrency(pair) == LeftCurrency(secondPair) || RightCurrency(pair) == RightCurrency(secondPair))
	{
		return true;
	}
	return false;
}

string CTree::LeftCurrency(string pair)
{
	if (pair.size() >= 6)
	{
		return pair.substr(0, 3);
	}
	return string("");
}

string CTree::RightCurrency(string pair)
{
	if (pair.size() >= 6)
	{
		return pair.substr(3, 3);
	}
	return string("");
}

bool CTree::GetClosestPaths(shared_ptr<CNode> node, vector<vector<string>> &paths, vector<string> path)
{
	vector<string> path_loc;

	if (CurrencyOnLeft(node->Pair, m_Currency) || CurrencyOnRight(node->Pair, m_Currency))
	{
		path.insert(path.end(), node->Pair);
		paths.insert(paths.end(), path);
		return true;
	}
	for (vector<shared_ptr<CNode>>::iterator i = node->LeftPairs.begin(); i != node->LeftPairs.end(); i++)
	{
		path_loc = path;
		path_loc.insert(path_loc.end(), node->Pair);
		if (GetClosestPaths((*i), paths, path_loc))
		{
			return true;
		}
	}
	for (vector<shared_ptr<CNode>>::iterator i = node->RightPairs.begin(); i != node->RightPairs.end(); i++)
	{
		path_loc = path;
		path_loc.insert(path_loc.end(), node->Pair);
		if (GetClosestPaths((*i), paths, path_loc))
		{
			return true;
		}
	}
	return false;
}

vector<string> CTree::GetClosestPath()
{
	vector<vector<string>> paths;
	vector<string> path;
	size_t min = 10;

	GetClosestPaths(Root, paths, path);
	for (vector<vector<string>>::iterator i = paths.begin(); i != paths.end(); i++)
	{
		if ((*i).size() < min)
		{
			path = (*i);
			min = i->size();
		}
	}

	return path;
}