/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#include "InternalCppDateTime.h"
#include "../HelperFunctions/HelperFunctions.h"

IDateTime::IDateTime()
{

}

bool IDateTime::operator<(const IDateTime &other)
{
	return m_Time < other.m_Time;
}

bool IDateTime::operator>(const IDateTime &other)
{
	return m_Time > other.m_Time;
}

IDateTime::IDateTime(const IDateTime &other)
{
	m_Time = other.m_Time;
}

IDateTime IDateTime::TimeCurrent()
{
	IDateTime t;

	t.m_Time = m_CurrentTime;

	return t;
}

IString IDateTime::GetDay(IDateTime &time)
{
	tm tmt;
	
	gmtime_s(&tmt, &(time.m_Time));
	
	switch (tmt.tm_wday)
	{
	case 0:
		return IString("Sunday");

	case 1:
		return IString("Monday");

	case 2:
		return IString("Thursday");

	case 3:
		return IString("Wensday");

	case 4:
		return IString("Thurdays");

	case 5:
		return IString("Friday");

	case 6:
		return IString("Saturday");

	default:
		return IString("Sunday");
	}
	return IString("Sunday");
}

void IDateTime::Assign(const IDateTime &other)
{
	m_Time = other.m_Time;
}

int IDateTime::GetHour()
{
	tm tmt;

	gmtime_s(&tmt, &(m_Time));

	return tmt.tm_hour;
}

bool IDateTime::IsTimeValid()
{
	return m_Time != 0;	
}

IDateTime IDateTime::AddMinute(IDateTime &time)
{
	IDateTime date;

	date.m_Time = time.m_Time + 60 * 1;

	return date;
}

IDateTime IDateTime::AddMinutes(IDateTime &time, int minutes)
{
	IDateTime date;

	date.m_Time = time.m_Time + 60 * minutes;

	return date;
}

IDateTime IDateTime::AddSeconds(IDateTime &time, int seconds)
{
	IDateTime date;

	date.m_Time = time.m_Time += seconds;

	return date;
}

int IDateTime::MinutesDiff(IDateTime &before, IDateTime &after)
{
	return (int)(difftime(after.m_Time, before.m_Time) / 60.0);
}

IDateTime IDateTime::MinusHours(IDateTime &time, int hoursAm)
{
	IDateTime date;

	date.m_Time = time.m_Time - hoursAm * 60 * 60;

	return date;
}

double IDateTime::SecondsDiff(const IDateTime &before, const IDateTime &after)
{
	return (int)(difftime(after.m_Time, before.m_Time));
}

IString IDateTime::FromSeconds(double seconds)
{
	int sec = (int)(seconds);
	int min = (int)(seconds / 60.0);
	int hour = (int)(seconds / (60.0*60.0));

	sec -= min * 60;
	min -= hour * 60;

	return IString(hour) + ":" + IString(min) + ":" + IString(sec);
}

IString IDateTime::TimeCurrentAsString()
{
	IDateTime time(TimeCurrent());

	return time.TimeAsString();
}

IString IDateTime::TimeAsString()
{
	IString str(CHelperFunctions::TimeAsString(m_Time).c_str());

	return str;
}

IDateTime IDateTime::FromString(IString &str)
{
	IDateTime time;

	time.m_Time = CHelperFunctions::GetTime(string(str.GetPlatformString()));
		
	return time;
}

time_t IDateTime::GetPlatformTime()
{
	return m_Time;
}

bool IDateTime::AreEqual(IDateTime &a, IDateTime &b)
{
	return a.m_Time == b.m_Time;
}

void IDateTime::SetCurrentTime(time_t t)
{
	m_CurrentTime = t;
}

IDateTime IDateTime::GetCurrentTime()
{
	IDateTime t;

	t.m_Time = m_CurrentTime;

	return t;
}

time_t IDateTime::GetCurrentPlatformTime()
{
	return m_CurrentTime;
}

double IDateTime::HoursDiffWithoutWeekends(const IDateTime& before, const IDateTime& after)
{
	IDateTime date = before;
	double seconds = IDateTime::SecondsDiff(before, after);

	if (seconds > 60.0 * 60.0 * 24.0 * 2.0)
	{
		seconds = 0.0;
		do
		{
			if (IString::AreEqual(IDateTime::GetDay(date), IString("Saturday")) || IString::AreEqual(IDateTime::GetDay(date), IString("Sunday")))
			{
				seconds += 60.0 * 60.0 * 24.0;
			}
			date = IDateTime::AddMinutes(date, 24 * 60);
		} while (date < after);
		seconds = IDateTime::SecondsDiff(before, after) - seconds;
		seconds -= IDateTime::SecondsDiff(after, date);
	}
	return seconds / (60.0 * 60.0);
}

time_t IDateTime::m_CurrentTime;