/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#include "InternalCppFileReader.h"
#include "InternalCppFileWriter.h"

void IFileReader::Initialize(IString name)
{
	m_Stream.open(IFileWriter::GetDirFiles() + string("\\") + string(name.GetPlatformString()).c_str(), ios::in | ios::app);
}

IString IFileReader::ReadLine()
{
	std::string str;
	IString ret;

	m_Stream >> str;
	ret.AssignString(str.c_str());

	return ret;	
}

IFileReader::~IFileReader()
{
	m_Stream.close();
}

bool IFileReader::IsEnd()
{
	return m_Stream.eof();
}

int IFileReader::FileSize()
{
	int pos = (int)m_Stream.tellg();
	int size;

	m_Stream.seekg(0, ios_base::end);
	size = (int)m_Stream.tellg();
	m_Stream.seekg(pos, ios_base::beg);
	
	return size;
}

void IFileReader::Seek(double percent)
{
	int size = FileSize();
	int pos = (int)(percent * size);

	m_Stream.seekg(pos, ios_base::beg);
}

void IFileReader::SeekBeg()
{
	m_Stream.seekg(0, ios_base::beg);
}

void IFileReader::SeekFromEnd(int bytesAm)
{
	double size = FileSize();
	int pos = (int)(size - bytesAm);

	if (pos > 0)
	{
		m_Stream.seekg(pos, ios_base::beg);
	}
}