/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#include "InternalCppInteroperability.h"
#include "../HelperFunctions/HelperFunctions.h"

void IInteroperability::Activate()
{
	
}

void IInteroperability::Assign(CMarket& market, CValuesProvider& provider)
{
	m_Market = &market;
	m_ValuesProvider = &provider;
	m_Market->AddValuesProvider(provider);
}

void IInteroperability::Print(const IString &str)
{
	cout << CHelperFunctions::TimeAsString(IDateTime::GetCurrentPlatformTime()) << " " << str.GetPlatformString() << endl;
}

void IInteroperability::RefreshRates()
{

}

unsigned long IInteroperability::GetLastError()
{
	return 0;
}

int IInteroperability::AccountLeverage()
{
	return 1;
}

const char* IInteroperability::AccountCurrency()
{
	return m_ValuesProvider->GetAccountCurrency().c_str();
}

bool IInteroperability::OrderSelect(int nr, unsigned long ticket)
{
	return false;
}

unsigned long IInteroperability::HistoryOrderTicket(int nr)
{
	return 0;
}

IDateTime IInteroperability::OrderOpenTime(unsigned long ticket, IDateTime time)
{
	IDateTime res;

	return res;
}

double IInteroperability::OrderProfit(unsigned long ticket)
{
	return m_Market->OrderProfit(ticket);
}

double IInteroperability::OrderOpenPrice(unsigned long ticket)
{
	return m_Market->OrderOpenPrice(ticket);
}

double IInteroperability::OrderCurrentPrice(unsigned long ticket)
{
	return m_Market->OrderCurrentPrice(ticket);
}

double IInteroperability::OrderTakeProfit(unsigned long ticket)
{
	return m_Market->OrderTakeProfit(ticket);
}

double IInteroperability::OrderStopLoss(unsigned long ticket)
{
	return m_Market->OrderStopLoss(ticket);
}

bool IInteroperability::CloseOrder(unsigned long ticket)
{
	return m_Market->CloseOrder(ticket);
}

double IInteroperability::ProfitAll(IGenericArray<int> &numbers, IGenericArray<double> &profits)
{
	double all = 0.0;

	for (int i = 0; i < m_Market->AmountOrders(); i++)
	{
		unsigned long nr = m_Market->OrderNr(i);
		double profit = m_Market->OrderProfit(nr);

		all += profit;		
		profits.Add(profit);
		numbers.Add((int)(nr));
	}
	return all;
}

void IInteroperability::CloseAll()
{
	m_Market->CloseAll();
}

void IInteroperability::CloseAll(ISortableArray<int> *trades)
{
	for (int i = 0; i < trades->Size(); i++)
	{
		m_Market->CloseOrder(trades->GetPos(i));
	}
}

double IInteroperability::AccountBalance()
{
	return m_Market->AccountBalance();
}

double IInteroperability::GetPriceAtTime(IString symbol, IDateTime date)
{
	return m_ValuesProvider->ValueOfColumnInSpecificTime(string(symbol.GetPlatformString()), date.GetPlatformTime());
}

double IInteroperability::GetPriceAtTimeHigh(IString symbol, IDateTime date)
{
	return m_ValuesProvider->ValueOfColumnInSpecificTime(string(symbol.GetPlatformString()), date.GetPlatformTime());
}

double IInteroperability::Spread(IString symbol)
{
	return 0.0;
}

double IInteroperability::BuyPrice(IString symbol)
{
	return m_ValuesProvider->ValueOfColumnAtCurrentTime(string(symbol.GetPlatformString()));
}

double IInteroperability::SellPrice(IString symbol)
{
	return m_ValuesProvider->ValueOfColumnAtCurrentTime(string(symbol.GetPlatformString()));
}

int IInteroperability::OrderSend(IString symbol, bool buy, double lotSize, double price, int slippage, double stopLoss, double takeProfit)
{
	return m_Market->OrderSend(symbol, buy, lotSize, price, stopLoss, takeProfit);
}

int IInteroperability::OrdersHistoryTotal()
{
	return 0;
}

double IInteroperability::MathAbs(double val)
{
	return abs(val);
}

double IInteroperability::MathExp(double val)
{
	return exp(val);
}

void IInteroperability::EventSetTimer(int secs)
{

}

void IInteroperability::EventKillTimer()
{

}

bool IInteroperability::MathIsValidNumber(double val)
{
	if (isnan(val))
	{
		return false;
	}
	else
	{
		return true;
	}
}

int IInteroperability::MathCeil(double val)
{
	return (int)ceil(val);
}

void IInteroperability::MathSrand()
{
	srand((unsigned int)(time(0)));
}

int IInteroperability::MathRand()
{
	int val = rand();

	if (val > 32767)
	{
		val = val % 32767;
	}
	return val;
}

IString IInteroperability::DoubleToString(double val)
{
	string str = to_string(val);

	return IString(str.c_str());
}

IString IInteroperability::IntegerToString(int val)
{
	string str = to_string(val);

	return IString(str.c_str());
}

IString IInteroperability::BoolToString(bool val)
{
	if (val)
	{
		IString str("true");

		return str;
	}
	else
	{
		IString str("false");

		return str;
	}
}

int IInteroperability::SymbolsTotal(bool shown)
{
	return m_ValuesProvider->ColumnsTotal();
}

IString IInteroperability::SymbolName(int nr, bool shown)
{
	return IString(m_ValuesProvider->ColumnName(nr).c_str());
}

double IInteroperability::NormalizeDouble(double price, int digits)
{
	return price;
}

int IInteroperability::Digits()
{
	return 5;
}

double IInteroperability::StringToDouble(IString &str)
{
	return stod(str.GetPlatformString());
}

IDateTime IInteroperability::StringToTime(IString &str)
{
	return IDateTime::FromString(str);
}

void IInteroperability::CheckMarket()
{
	m_Market->CheckOrders();
}

IString IInteroperability::TimeToString(IDateTime& time)
{
	return time.TimeAsString();
}