/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_INTERNAL_CPP_INTEROPERABILIT
#define FILE_INTERNAL_CPP_INTEROPERABILIT

#include "../../../Interoperability/GenericArray.h"
#include "../../../Interoperability/SortableArray.h"
#include "InternalCppDateTime.h"
#include "InternalCppString.h"
#include "../ValuesProvider/ValuesProvider.h"
#include "../Market/Market.h"
#include <string>
#include <math.h>
#include <time.h>
#include <iostream>

using namespace std;

//+------------------------------------------------------------------+
class IInteroperability
{
private:
	CValuesProvider *m_ValuesProvider;
	CMarket *m_Market;

public:
	void Activate();

	void Assign(CMarket& market, CValuesProvider& provider);

	void Print(const IString &str);

	void RefreshRates();

	unsigned long GetLastError();

	int AccountLeverage();

	const char* AccountCurrency();

	bool OrderSelect(int nr, unsigned long ticket);

	unsigned long HistoryOrderTicket(int nr);

	IDateTime OrderOpenTime(unsigned long ticket, IDateTime time);

	double OrderProfit(unsigned long ticket);

	double OrderOpenPrice(unsigned long ticket);

	double OrderCurrentPrice(unsigned long ticket);

	double OrderTakeProfit(unsigned long ticket);

	double OrderStopLoss(unsigned long ticket);

	bool CloseOrder(unsigned long ticket);

	double AccountBalance();

	double GetPriceAtTime(IString symbol, IDateTime date);

	double GetPriceAtTimeHigh(IString symbol, IDateTime date);

	double Spread(IString symbol);

	double BuyPrice(IString symbol);

	double SellPrice(IString symbol);

	int OrderSend(IString symbol, bool buy, double lotSize, double price, int slippage, double stopLoss, double takeProfit);

	int OrdersHistoryTotal();

	double ProfitAll(IGenericArray<int> &numbers, IGenericArray<double> &profits);

	void CloseAll();

	void CloseAll(ISortableArray<int> *trades);

	double MathAbs(double val);

	double MathExp(double val);

	void EventSetTimer(int secs);

	void EventKillTimer();

	bool MathIsValidNumber(double val);

	int MathCeil(double val);

	void MathSrand();

	int MathRand();

	IString DoubleToString(double val);

	IString IntegerToString(int val);

	IString BoolToString(bool val);

	int SymbolsTotal(bool shown);

	IString SymbolName(int nr, bool shown);

	double NormalizeDouble(double price, int digits);

	int Digits();

	double StringToDouble(IString &str);

	IDateTime StringToTime(IString &str);

	IString TimeToString(IDateTime &time);

	void CheckMarket();

};
//+------------------------------------------------------------------+
#endif
