#include "Market.h"
#include <algorithm>
#include "../HelperFunctions/HelperFunctions.h"
#include "../Internal/InternalCppDateTime.h"

CMarket::CMarket()
{
	m_Writer.Initialize(IString("Closed_Trades.txt"));
	m_Balance = 100000.0;
	m_Invested = 0.0;
}

bool COrder::compare(COrder& i1, COrder& i2)
{
	return (i1.nr < i2.nr);
}

double COrder::Profit(CValuesProvider *vp)
{
	double lots = lotSize / 0.01;

	return lots * vp->Multiplicator(symbol) * 1000.0 - cost;
}

double COrder::Cost(CValuesProvider *vp)
{
	double lots = lotSize / 0.01;

	return lots * vp->Multiplicator(symbol) * 1000.0;
}

time_t CMarket::OrderOpenTime(unsigned long ticket)
{
	vector<COrder>::iterator order;

	if (Find(ticket, order))
	{
		return order->openTime;
	}
	return 0;
}

double CMarket::OrderProfit(unsigned long ticket)
{
	vector<COrder>::iterator order;

	if (Find(ticket, order))
	{
		return order->Profit(m_ValuesProvider);
	}
	return 0.0;
}

double CMarket::OrderOpenPrice(unsigned long ticket)
{
	vector<COrder>::iterator order;

	if (Find(ticket, order))
	{
		return order->price;
	}
	return 0.0;
}

double CMarket::OrderCurrentPrice(unsigned long ticket)
{
	return 0.0;
}

double CMarket::OrderTakeProfit(unsigned long ticket)
{
	vector<COrder>::iterator order;

	if (Find(ticket, order))
	{
		return order->takeProfit;
	}
	return 0.0;
}

double CMarket::OrderStopLoss(unsigned long ticket)
{
	vector<COrder>::iterator order;

	if (Find(ticket, order))
	{
		return order->stopLoss;
	}
	return 0.0;
}

bool CMarket::CloseOrder(unsigned long ticket)
{
	vector<COrder>::iterator order;
	double profit;

	if (Find(ticket, order))
	{
		string line;
		string buy;

		if (order->buy)
		{
			buy = "Buy";
		}
		else
		{
			buy = "Sell";
		}

		profit = order->Profit(m_ValuesProvider);
		m_Balance += profit;
		m_Invested -= (order->cost + profit);
		line = "Symbol:" + order->symbol + ", Buy:" + buy + ", Price:" + to_string(order->price) + ", Cost:" + to_string(order->cost) + ", TakeProfit:" + to_string(order->takeProfit) + ", Stop Loss:" + to_string(order->stopLoss) + ", Lot Size:" + to_string(order->lotSize) + ", Profit:" + to_string(profit) + ", Open Time:" + CHelperFunctions::TimeAsString(order->openTime) + ", Close Time:" + CHelperFunctions::TimeAsString(IDateTime::GetCurrentPlatformTime());
		m_Writer.Write(IString(line.c_str()));

		m_Orders.erase(order);
		return true;
	}
	return false;
}

int CMarket::AmountOrders()
{
	return (int)(m_Orders.size());
}

unsigned long CMarket::OrderNr(int nr)
{
	return m_Orders[nr].nr;
}

bool CMarket::Find(unsigned long ticket, vector<COrder>::iterator& order)
{
	order = find_if(m_Orders.begin(), m_Orders.end(), [&ticket](COrder i) { return i.nr == ticket; });

	if (order != m_Orders.end())
	{
		return true;
	}
	return false;
}

int CMarket::OrderSend(IString symbol, bool buy, double lotSize, double price, double stopLoss, double takeProfit)
{
	COrder order;
	
	order.buy = buy;
	order.lotSize = lotSize;
	order.price = price;
	order.stopLoss = stopLoss;
	order.takeProfit = takeProfit;
	order.symbol = string(symbol.GetPlatformString());
	order.openTime = m_ValuesProvider->CurrentTime();

	order.nr = Nr;
	Nr++;
	order.cost = order.Cost(m_ValuesProvider);

	if (m_Balance >= m_Invested + order.cost)
	{
		m_Invested += order.cost;
		m_Orders.insert(m_Orders.end(), order);
		cout << "Order Send, Symbol :" + order.symbol + ", Cost : " + to_string(order.cost) + ", Invested : " + to_string(m_Invested) + ", Balance : " + to_string(m_Balance) << endl;
		return order.nr;
	}
	else
	{
		cout << "Order Not Send, Symbol :" + order.symbol + ", Cost : " + to_string(order.cost) + ", Invested : " + to_string(m_Invested) + ", Balance : " + to_string(m_Balance) << endl;
		return -1;
	}
}

void CMarket::CheckOrders()
{
	vector<unsigned long> orders;

	for (vector<COrder>::iterator i = m_Orders.begin(); i != m_Orders.end(); i++)
	{
		double price = m_ValuesProvider->ValueOfColumnAtCurrentTime(i->symbol);

		if (i->buy && price >= i->takeProfit ||
			i->buy && price <= i->stopLoss ||
			!i->buy && price <= i->takeProfit ||
			!i->buy && price >= i->stopLoss)
		{
			orders.insert(orders.end(), i->nr);
		}
	}
	for (vector<unsigned long>::iterator i = orders.end(); i != orders.begin(); i--)
	{
		CloseOrder((*(i - 1)));
	}
}

void CMarket::AddValuesProvider(CValuesProvider& provider)
{
	m_ValuesProvider = &provider;
}

double CMarket::AccountBalance()
{
	return m_Balance;
}

void CMarket::CloseAll()
{
	vector<unsigned long> orders;

	for (vector<COrder>::iterator i = m_Orders.begin(); i != m_Orders.end(); i++)
	{
		orders.insert(orders.end(), i->nr);
	}
	for (vector<unsigned long>::iterator i = orders.end(); i != orders.begin(); i--)
	{
		CloseOrder((*(i - 1)));
	}
}

unsigned long CMarket::Nr = 1;