#include "ValuesProvider.h"
#include <sstream>
#include <fstream>
#include "../HelperFunctions/HelperFunctions.h"
#include "../HelperFunctions/Node.h"
#include <iostream>
#include "../Internal/InternalCppDateTime.h"

void CValuesProvider::Init(string filename)
{
	string line;
	ifstream infile;
	m_RowNr = 0L;

	infile.open(filename);
	infile.seekg(0);
	if (infile.is_open())
	{
		cout << "Opened file : " + filename << endl;
		while (getline(infile, line))
		{
			ParseLine(line);
		}
		cout << "Readed " << m_ColumnsNames.size() << " Currency Pairs." << endl;
		cout << "Readed " << m_Rows.size() << " time entries." << endl;
		cout << "Begin Time : " << CHelperFunctions::TimeAsString(m_Rows[0].GetDate()) << endl;
		cout << "End Time : " << CHelperFunctions::TimeAsString(m_Rows[m_Rows.size() - 1].GetDate()) << endl;
		PreparePaths(m_Currency);
		IDateTime::SetCurrentTime(m_Rows[m_RowNr].GetDate());
	}
}

void CValuesProvider::PreparePaths(string currency)
{
	for (vector<string>::iterator i = m_ColumnsNames.begin(); i != m_ColumnsNames.end(); i++)
	{
		CTree tree;
		vector<string> path;

		tree.Init(*i, m_ColumnsNames, currency);
		path = tree.GetClosestPath();

		for (vector<string>::iterator j = path.begin(); j != path.end(); j++)
		{
			m_Map[(*i)].PathStr.insert(m_Map[(*i)].PathStr.end(), *j);
		}
	}
}

void CValuesProvider::ParseLine(string line)
{
	istringstream f(line);
	string s;
	vector<string> vec;
	time_t time;
	int j;

	while (getline(f, s, ';'))
	{
		vec.insert(vec.end(), s);
	}
	time = CHelperFunctions::GetTime(vec[0]);
	if (vec[1][0] >= 'a' && vec[1][0] <= 'z' || vec[1][0] >= 'A' && vec[1][0] <= 'Z')
	{
		m_ColumnsNames.clear();
		m_Map.clear();
		j = 0;
		for (int i = 1; i < vec.size(); i++)
		{
			if (find(m_ColumnsNames.begin(), m_ColumnsNames.end(), vec[i]) == m_ColumnsNames.end())
			{
				m_ColumnsNames.insert(m_ColumnsNames.end(), vec[i]);
				m_Map[vec[i]].Nr = j;
				j++;
			}
		}
	}
	else
	{
		CRow row;
		double val;

		row.Init((int)(vec.size() - 1));
		row.SetDate(time);
		for (int i = 1; i < vec.size(); i++)
		{
			val = stod(vec[i]);
			row.SetColumnValue(i - 1, val);
		}
		m_Rows.insert(m_Rows.end(), row);
	}
}

double CValuesProvider::ValueOfColumnAtCurrentTime(string column)
{
	return m_Rows[m_RowNr].GetColumnValue(m_Map[column].Nr);
}

double CValuesProvider::ValueOfColumnInSpecificTime(string column, time_t time)
{
	vector<CRow>::iterator before, after;
	time_t curr, bef, aft;

	curr = time;

	after = lower_bound(m_Rows.begin(), m_Rows.end(), curr, [](CRow& row, time_t value) { return row.GetDate() < value; });

	if (after != m_Rows.end() && after != m_Rows.begin())
	{
		before = after - 1;
		bef = before->GetDate();
		aft = after->GetDate();

		return (before->GetColumnValue(m_Map[column].Nr)) * (aft - curr) / ((double)(aft - bef)) + (after->GetColumnValue(m_Map[column].Nr)) * (curr - bef) / ((double)(aft - bef));
	}
	return 0.0;
}

string CValuesProvider::ColumnName(int nr)
{
	for (map<string, CPairInfo>::iterator i = m_Map.begin(); i != m_Map.end(); i++)
	{
		if (i->second.Nr == (size_t)nr)
		{
			return i->first;
		}
	}
	return string("");
}

int CValuesProvider::ColumnsTotal()
{
	return (int)(m_ColumnsNames.size());
}

bool CValuesProvider::AssignNextRow()
{
	if (m_RowNr < m_Rows.size() - 1)
	{
		time_t t;

		m_RowNr++;
		t = m_Rows[m_RowNr].GetDate();
		IDateTime::SetCurrentTime(t);
		return true;
	}
	else
	{
		return false;
	}
}

void CValuesProvider::AddToCurrentRow(double hours)
{
	time_t t = CHelperFunctions::AddHours(m_Rows[0].GetDate(), hours);

	for (int i = 0; i < m_Rows.size(); i++)
	{
		if (m_Rows[i].GetDate() > t)
		{
			m_RowNr = i;
			t = m_Rows[m_RowNr].GetDate();
			IDateTime::SetCurrentTime(t);
			return;
		}
	}
}

time_t CValuesProvider::CurrentTime()
{
	return IDateTime::GetCurrentPlatformTime();
}

void CValuesProvider::SetAccountCurrency(string currency)
{
	m_Currency = currency;
}

string CValuesProvider::GetAccountCurrency()
{
	return m_Currency;
}

double CValuesProvider::Multiplicator(string currency, string pair, string& ocurrency)
{
	string left = CTree::LeftCurrency(pair);
	string right = CTree::RightCurrency(pair);

	if (m_Currency == currency && currency == left)
	{
		ocurrency = left;
		return 1.0;
	}
	else if (m_Currency == currency && currency == right)
	{
		ocurrency = right;
		return m_Rows[m_RowNr].GetColumnValue(m_Map[pair].Nr);
	}
	else if (currency == left)
	{
		ocurrency = right;
		return 1.0 / m_Rows[m_RowNr].GetColumnValue(m_Map[pair].Nr);
	}
	else
	{
		ocurrency = left;
		return m_Rows[m_RowNr].GetColumnValue(m_Map[pair].Nr);
	}
}

double CValuesProvider::Multiplicator(string pair)
{
	double multi = 1.0;
	string ocurr = m_Currency;

	for (vector<string>::iterator i = m_Map[pair].PathStr.begin(); i != m_Map[pair].PathStr.end(); i++)
	{
		multi *= Multiplicator(ocurr, *i, ocurr);
	}
	return multi;
}

void CRow::Init(int am)
{
	m_ColumnsValues.resize(am);
}

time_t CRow::GetDate()
{
	return m_RowDate;
}

void CRow::SetDate(time_t time)
{
	m_RowDate = time;
}

double CRow::GetColumnValue(int nr)
{
	return m_ColumnsValues[nr];
}

void CRow::SetColumnValue(int nr, double val)
{
	m_ColumnsValues[nr] = val;
}