/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_CONFIG
#define FILE_CONFIG

#include "../../Interoperability/String.h"

//+------------------------------------------------------------------+
enum MethodDCombinationType{
	BEST,
	WORST,
	BEST_COMMON,
	WORST_COMMON
};

enum MultivariateTrailStop {
	TRAIL_STOP,
	SPEED_STOP,
	SURFING_STOP,
	SURFING_PROFIT,
	NONE,
	SURFING_REVERSE_PROFITS,
	SURFING_REVERSE_BALANCE,
	SURFING_REVERSE_BALANCE_SPEED
};

class CConfig {
public:
	/*Blocks Robot run if Levarage is to big*/
	const static bool CheckLeverage;
	/*Lot size*/
	const static double MaxVolume;
	/*Maximum Lots amount that can be Traded*/
	const static int MaxAmount;
	/*Stops Robot run if Account Balance to Large*/
	const static bool CheckBalance;
	/*Lot size conversion to real*/
	const static double MinVolume;
	/*Value of slpppage for Ordersend*/
	const static int SlipPage;
	/*Reversing all Trades in last algorithm Step*/
	const static bool ReverseTrades;
	/*Lot size to use for every Trade*/
	const static double LotSize;
	/*Determines how often Robot checks all past trades to determine if reverse trades for future trades*/
	const static int ReverseTradesPastHoursAmount;
	/*If to determine reverse trades based on past trades*/
	const static bool UseReverseTradesPastHours;
	/*If to use Neural Networks in for all Sell/Buy operations*/
	static bool UseSellBuyNeuralNetworks;
	/*If to use Neural Networks in for all Symbols*/
	const static bool UseIndividualNeuralNetworks;
	/*If to use trades that were really performed, logic of this is connected to if ReverseTrades was set to true*/
	const static bool UsePerformedTradesForReverseTradesPastHours;
	/*If to Analyse Trend present in log files*/
	const static bool UseTrendFromLogs;
	/*If to use reverse logic in Method D*/
	const static bool UseAfterTrend;
	/*Specifies if to determine values of params used in Method D by analysis of logs*/
	const static bool UseAfterTrendAutomaticParams;
	/*Specifies if to use Method E logic of using rules determining near past robot performence*/
	const static bool UseRules;
	/*Specifies if to Calculate Distances to last Minimas, Maximas and use Method F*/
	const static bool UseCalculateDistance;
	/*Amount of hours passed to trigger MethodC*/
	const static int TimeInHoursForTrendFromLogs;
	/*Amount of finished trades to analyze in Method D*/
	const static int AfterTrendHoursToAnalysisLength;
	/*Specifies under which condition automatically close all open trades in Method G*/
	const static MultivariateTrailStop ConditionMultivariateTrailStop;

	/*Specifies maximum amount of data that will be taken to analysis from last log file*/
	const static double TrendAnalysisMaxLogFileSize;

	/*Value of proportional change between established Minimum, Maximum to consider point as next Minimum, Maximum */
	const static double PercentageOfChange;
	/*Specifies value by which diffrence between take profit and stop loss can be multiplied, this vvalue is used just before placing order*/
	const static double TakeProfitStopLossBiggerSpan;

	/*How often main timer function is called*/
	const static int TimerSecondsAmount;
	/*Historical data interval*/
	const static int MinutesSpan;

	/*Log all log messages*/
	const static bool HeavyLoggingInFile;

	/*How many hours of historical data will be traied to be accessed after every re-initialization*/
	const static int PastHoursCount;
	/*After how many hours elapsed there will be Robot re-initialization*/
	const static int FutureHoursWindow;
	/*How many hours of accessed historical data use to do analysis*/
	const static int PastHoursCountForAnalysis;
	/*If both Neural Networks will be unable to train, after how many hours to re-initialize Robot*/
	const static double FutureHoursInvalidRestart;
	/*Interval of writing to file values of selected Symbols*/
	const static int FarPastDataIntervalSeconds;
	/*Interval of writing to file values of Profit*/
	const static int PastProfitIntervalSeconds;
	/*If to Write Far Past Data*/
	const static bool WriteFarPastData;
	/*If to Write Profit Values*/
	const static bool WritePastProfit;
	/*Minimal amount of past trades extracted from logs to proceed with Method C*/
	const static int TradesAmountMin;
	/*Max diffrence in amount of determined Tradable Symbols determined for current period to be considered when extracting past trades from logs*/
	const static int TradableAmountMaxDiff;
	/*Min lenght in Days of Past Data*/
	const static int MinPastDataLength;
	/*Max length in Days of Past Data*/
	const static int MaxPastDataLength;
	/*Minimal value of percentage difference between positive correlations and negative correlations in Method F to proceed with Trade*/
	const static double MinPercentageDistanceDifference;
	/*Specifies if to use distance provider solution from positive correlations*/
	const static bool ConsiderPositiveCorrelation;
	/*Specifies if to use distance provider solution from negative correlations*/
	const static bool ConsiderNegativeCorrelation;
	/*Specifies after what value of lowering price of all open trades, all open trades are beeing closed*/
	const static double MultivariateTrailStopValue;
	/*Specifies if to use multivariate trail stop minimum value*/
	const static bool MultivariateTrailStopMinimum;
	/*Specifies after what value of profit gain/loss speed, (all open)/(open) trades are beeing closed*/
	const static double MultivariateTrailSpeedValue;
	/*Specifies after what value of profit gain/loss speed, reverse open trades are beeing closed(used when TRAIL_SURFING)*/
	const static double MultivariateReverseTrailSpeedValue;
	/*Specifies in what time frame speed of profit gain/loss is calculated*/
	const static double MultivariateTrailSpeedHours;
	/*Specifies how often log current speeds of gain/loss*/
	const static int TrailSpeedLogCountDown;
	/*Minimal value of profit to close all trades*/
	const static double MultivariateProfitStopMin;
	/*Maximal value of profit to close all trades*/
	const static double MultivariateProfitStopMax;
	/*Minimal value of reverse trades profit to close all trades*/
	const static double MultivariateReverseProfitStopMin;
	/*Maximal value of reverse trades profit to close all trades*/
	const static double MultivariateReverseProfitStopMax;
	/*Minimal value of sum of all profits after which trades will be reversed*/
	const static double MultivariateReverseTradesMin;
	/*Minimal value of balance beeing lowered after which trades will be reversed*/
	const static double MultivariateBalanceReverseTradesMin;
	/*Minimal value of speed og balance gain/loss to close all open trades and reverse trade future trades*/
	const static double MultivariateSurfingReverseBalanceSpeedMin;
	/*Value of hours used for calculating speed of balance gain/loss for closing and reversing trades*/
	const static double MultivariateSurfingReverseBalanceSpeedHours;

	/*Minimum value of TakeProfit Multi*/
	const static double PossibleProfitMultiMin;
	/*Minimum value of StopLoss Multi*/
	const static double StopLossMultiMin;
	/*Maximum value of TakeProfit Multi*/
	const static double PossibleProfitMultiMax;
	/*Maximum value of StopLoss Multi*/
	const static double StopLossMultiMax;
	/*Value of next step of change of both of the Multis*/
	const static double CheckMultiStep;

	/*How many unsuccessfull trades in a row to stop trading*/
	const static int FailedInRow;
	/*How many successfull trades in the row to start trading*/
	const static int SuccededInRow;
	/*How many unsuccessfull trades in a row to stop trading symbol*/
	const static int FailedInRowSingle;
	/*How many successfull trades in the row to start trading symbol*/
	const static int SuccededInRowSingle;
	/*Count each succeeded and failed for every symbol individually*/
	const static bool UseSingleInRow;
	/*Specifies if to bypass logic of determining in there was enought successffullInRow to proceed with trade*/
	const static bool ByPassInRowCheck;

	/*Part of existing Balance to be used in Trading*/
	const static double PercentOfBalanceToTrade;
	/*Lern Neural Networks by examples from percent of Symbols that were determined to be most profitable in analysis of historical data*/
	const static double PercentOfTopCurrenciesToLearn;
	/*How many trades can be run at the same time*/
	const static int SingleTradesAmount;
	/*Which value of Symbol Price determines to use bigger Slip Page value*/
	const static double BiggerSlipPagePriceValue;
	/*Minimum ratio of Gains/Loses to Symbol be considered as taken into account*/
	const static double GainsMinPercentage;
	/*Percent of tradable Symbols from all Symbols that were used during training of the Neural Network*/
	const static double TradableOfLearnedPercent;
	/*Test set is last part of all examples determined during analysis*/
	const static bool DividedSets;
	/*How many determined as passed to all passed in test vectors*/
	const static double PercentPassedToAllPassedTest;
	/*How many determined as passed to all passed in learn vectors*/
	const static double PercentPassedToAllPassedLearn;
	/*How many times try to Learn Neural Network if Network is invalid*/
	const static int NeuralNetworkLearnAttempts;
	/*Successfull/Failed trades in last period, before re-initaialization*/
	const static double LastPeriodSuccessfullPercentMin;
	/*Analysing and Trading only on Currency Pairs;*/
	const static bool UseOnlyCurrencyPairs;
	/*Percent of Narrowing SP and TP*/
	const static double MultiPercent;
    /*Determine minimum amount of Tradable Symbols to Trade in given Period*/
    const static int MinTradableSymbols;
    /*Determines First minimal percent value of profit reached to log that it has happened*/
    const static double MinFirstPercentOfProfitReached;
	/*Determines Second minimal percent value of profit reached to log that it has happened*/
	const static double MinSecondPercentOfProfitReached;
	/*Determines Third minimal percent value of profit reached to log that it has happened*/
	const static double MinThirdPercentOfProfitReached;
	/*Determines Fourth minimal percent value of profit reached to log that it has happened*/
	const static double MinFourthPercentOfProfitReached;

	/*Proceed in executing trades, for testing purposes can be checked only log files if value set to false*/
	const static bool ExecuteTrades;
	/*Determines if to limit Currency Pairs with Account Currency only to Buying or Selling*/
	const static bool ConsiderAccountCurrency;
	/*How often modyfications to open orders are beeing done*/
	const static int IntervalDoModifications;
	/*How often Neural Network for Open Trades Modification is beeing learned*/
	const static int IntervalDoLearning;
	/*Specifies upper value between 0.0 and 1.0 determining propability of placing trade*/
	const static double ProceedWithTradeLimit;

	/*Percent of Take Profit change that Stop Loss can have*/
	const static double TakeProfitPercentForStopLoss;
	/*Determines if Normalize Profit*/
	const static bool UseMaxMultiTakeProfitStopLoss;
	/*Percent of Take Profit change that Stop Loss can have to Ignore the Order*/
	const static double TakeProfitPercentForStopLossToIgnore;
	/*Make TakeProfit delta equal StopLoss delata*/
	const static bool MakeStopLossEqualTakeProfit;
	/*Minimal required value of PMS*/
	const static double MinPMSValue;

	/*List of discoverable currencies*/
	static IGenericObjectArray<IString> Currencies;
	/*List of predefined trade rules*/
	static IGenericObjectArray<IString> Rules;
	/*Lot size if trading Symbols that are not Currency Pairs*/
	const static int LotSizeForNoCurrencies;
	/*Amount of hours to determine past speed of trades (SuccessfullTrades/LastHours)*/
	const static int LastHours;

	/*Length of Past Points used to train Neural Networks before established Minimum, Maximum*/
	const static double PastPointsFraction;
	/*Amount of Inputs in Neural Networks minus 2(for SL and TP value)*/
	const static int NetworkInputsPointsAmount;
	/*Neural Networks Learn parameter*/
	const static double LearnParameter;
	/*Percent of correctly classified Learn Vectors by Neural Network during Learning to stop Learning*/
	const static double LearnPercentReach;
	/*Percent of correctly classified Learn Vectors by Neural Network during Learning to be considered as valid Neural Network*/
	const static double LearnPercentReachMin;
	/*Amount of hidden layers in Neural Networks*/
	const static int HiddenLayersAmount;
	/*Dimension of Neurons Activation Function*/
	const static int ActivationFunctionDimension;
	/*Amount of hidden layers in Neural Networks used for separate Symbols*/
	const static int HiddenLayersAmountForSymbolNetwork;
	/*Dimension of Neurons Activation Function used for Trend Neural Network*/
	const static int ActivationFunctionDimensionForTrendNetwork;
	/*Amount of hidden layers in Neural Networks used for Trend Analysis*/
	const static int HiddenLayersAmountForTrendNetwork;
	/*Dimension of Neurons Activation Function used for separate Symbols*/
	const static int ActivationFunctionDimensionForSymbolNetwork;
	/*Max amount of epochs of Learning Neural Networks*/
	const static int MaxEpoch;
	/*Max amount of epoch to stop learning when results are constantly the same*/
	const static int MaxEpochSameValues;
	/*Percent of Vectors of all vectors got after analysis that be used as test vectors*/
	const static double TestVectorsPercent;
	/*Output value of Neuron in last layer to vector be considered as passed*/
	const static double ThresholdValue;
	/*Percent of correctly classified Test Vectors by Neural Network during Learning to be considered as valid Neural Network*/
	const static double TestPercentReachMin;
	/*Minimal amount of past trades in row to analyze in Method C*/
	const static int PastTradeMinAmount;
	/*Maximal amount of past trades in row to analyze in Method C*/
	const static int PastTradeMaxAmount;
	/*Amount of past trades in row to analyze in Method D*/
	const static int PastTradeWithRTMaxAmount;
	/*Min Amount of past trades in row to analyze in Method D*/
	const static int PastTradeWithRTMinAmount;
	/*Ratio of failed trades to switch RT flag in Method D*/
	const static double PastTradeWithRTSwitchRatio;
	/*Which option to choose when determining MethodD params automatically*/
	const static MethodDCombinationType PastTradeWithRTCombination;
	/*What percentage of other permutations of possible params combination to include in common combination of Method D*/
	const static double PastTradWithRTCommonCombinationIncludePercent;
	/*Automatic Params for Method D Minimal value of hours window*/
	const static int PastTradeWithRTAutomaticParamsHoursMin;
	/*Automatic Params for Method D Maximal value of hours window*/
	const static int PastTradeWithRTAutomaticParamsHoursMax;
	/*Automatic Params for Method D Minimal value of trades amount*/
	const static int PastTradeWithRTAutomaticParamsTradesMin;
	/*Automatic Params for Method D Maximal value of trades amount*/
	const static int PastTradeWithRTAutomaticParamsTradesMax;
	/*Automatic Params for Method D Minimal value of fail success Ratio*/
	const static double PastTradeWithRTAutomaticParamsSwitchRatioMin;
	/*Automatic Params for Method D Maximal value of fail success Ratio*/
	const static double PastTradeWithRTAutomaticParamsSwitchRatioMax;
	/*Automatic Params for Method D value of step for fail success Ratio*/
	const static double PastTradeWithRTAutomaticParamsSwitchRatioStep;
	/*Automatic Params Reverse Trade based if current choice is best*/
	const static bool PastTradeWithRTAutomaticReverseTrade;
	/*Specifies how many past log files are beeing analyzed for Method C*/
	const static int LogsAmountToAnalyze;
	/*Minimal value of Latest in row Past trades multiply increase*/
	const static double PastTradeLineMultiplyMin;
	/*Maximal value of Latest in row Past trades multiply increase*/
	const static double PastTradeLineMultiplyMax;
	/*Amount of steps of tested Past trades multiply increase*/
	const static int PastTradeLineMultiplyStepsAmount;
	/*Maximal amount of vectors of Past Trades to be analyzed in Method C*/
	const static int PastTradesVectorsMaxAmount;
	/*Determines if to use Neural Networks in Trend Analysis*/
	const static bool PastTrendsUseNeuralNetwork;

	static void Init()
	{
		Currencies.Resize(23);
		(*(Currencies.GetPointerToValue(0))).AssignString("AUD"); (*(Currencies.GetPointerToValue(1))).AssignString("CAD"); (*(Currencies.GetPointerToValue(2))).AssignString("CHF");
		(*(Currencies.GetPointerToValue(3))).AssignString("JPY"); (*(Currencies.GetPointerToValue(4))).AssignString("NZD"); (*(Currencies.GetPointerToValue(5))).AssignString("SGD");
		(*(Currencies.GetPointerToValue(6))).AssignString("USD"); (*(Currencies.GetPointerToValue(7))).AssignString("EUR"); (*(Currencies.GetPointerToValue(8))).AssignString("CZK");
		(*(Currencies.GetPointerToValue(9))).AssignString("DDK"); (*(Currencies.GetPointerToValue(10))).AssignString("GBP"); (*(Currencies.GetPointerToValue(11))).AssignString("HUF");
		(*(Currencies.GetPointerToValue(12))).AssignString("MXN"); (*(Currencies.GetPointerToValue(13))).AssignString("NOK"); (*(Currencies.GetPointerToValue(14))).AssignString("PLN");
		(*(Currencies.GetPointerToValue(15))).AssignString("SEK"); (*(Currencies.GetPointerToValue(16))).AssignString("TRY"); (*(Currencies.GetPointerToValue(17))).AssignString("ZAR");
		(*(Currencies.GetPointerToValue(18))).AssignString("CNH"); (*(Currencies.GetPointerToValue(19))).AssignString("HKD"); (*(Currencies.GetPointerToValue(20))).AssignString("NOK");
		(*(Currencies.GetPointerToValue(21))).AssignString("RUB"); (*(Currencies.GetPointerToValue(22))).AssignString("THB");

		Rules.Resize(2);
		(*(Rules.GetPointerToValue(0))).AssignString("MinSpeed:-1.0;MaxSpeed:1.0;PercentSuccessfull:70.0");
		(*(Rules.GetPointerToValue(1))).AssignString("MinSpeed:-1.0;MaxSpeed:1.0;PercentSuccessfull:30.0");
	}
};
const bool CConfig::CheckLeverage = false;
const bool CConfig::CheckBalance = true;

const double CConfig::MaxVolume = 1000.0;
const int CConfig::MaxAmount = 100;
const double CConfig::MinVolume = 0.01;
const double CConfig::LotSize = 0.01;
const int CConfig::SlipPage = 20;

bool CConfig::UseSellBuyNeuralNetworks = true; //Method A
const bool CConfig::UseIndividualNeuralNetworks = true; //Method B
const bool CConfig::UseTrendFromLogs = false; //Method C
const bool CConfig::UseAfterTrend = false; //Method D
const bool CConfig::UseAfterTrendAutomaticParams = false;
const bool CConfig::UseRules = true; //Method E
const bool CConfig::UseCalculateDistance = true; //Method F
const MultivariateTrailStop CConfig::ConditionMultivariateTrailStop = MultivariateTrailStop::NONE; //Method G
const int CConfig::TimeInHoursForTrendFromLogs = 5 * 24;
const int CConfig::AfterTrendHoursToAnalysisLength = 24;
const bool CConfig::MultivariateTrailStopMinimum = false;
const double CConfig::MultivariateTrailSpeedValue = 11.0;
const double CConfig::MultivariateReverseTrailSpeedValue = 11.0;
const double CConfig::MultivariateTrailSpeedHours = 2.0;
const int CConfig::TrailSpeedLogCountDown = 60;
const double CConfig::MultivariateProfitStopMin = -500.0;
const double CConfig::MultivariateProfitStopMax = 3.0;
const double CConfig::MultivariateReverseProfitStopMin = -500.0;
const double CConfig::MultivariateReverseProfitStopMax = 2.0;
const double CConfig::MultivariateReverseTradesMin = -50.0;
const double CConfig::MultivariateBalanceReverseTradesMin = 50.0;
const double CConfig::MultivariateSurfingReverseBalanceSpeedHours = 38.5;
const double CConfig::MultivariateSurfingReverseBalanceSpeedMin = -1.0;

const double CConfig::TrendAnalysisMaxLogFileSize = 2.5 * 1024.0 * 1024.0;

const int CConfig::TimerSecondsAmount = 1;
const int CConfig::MinutesSpan = 1;

const int CConfig::FutureHoursWindow = 6;
const double CConfig::FutureHoursInvalidRestart = 3.0;
const int CConfig::FarPastDataIntervalSeconds = 60;
const int CConfig::PastProfitIntervalSeconds = 1;
const bool CConfig::WriteFarPastData = true;
const bool CConfig::WritePastProfit = true;
const int CConfig::TradesAmountMin = 20;
const int CConfig::TradableAmountMaxDiff = 5;
const int CConfig::MinPastDataLength = 14;
const int CConfig::MaxPastDataLength = 14 * 4;

const double CConfig::PossibleProfitMultiMin = 0.0;
const double CConfig::StopLossMultiMin = 0.0;
const double CConfig::PossibleProfitMultiMax = 3.0;
const double CConfig::StopLossMultiMax = 3.0;
const double CConfig::CheckMultiStep = 0.03;

const int CConfig::FailedInRow = 0;
const int CConfig::SuccededInRow = 0;
const int CConfig::FailedInRowSingle = 0;
const int CConfig::SuccededInRowSingle = 0;
const bool CConfig::UseSingleInRow = false;
const bool CConfig::ByPassInRowCheck = true;

const bool CConfig::ReverseTrades = false;

const double CConfig::MinPercentageDistanceDifference = 20.0;
const bool CConfig::ConsiderPositiveCorrelation = true;
const bool CConfig::ConsiderNegativeCorrelation = true;
const double CConfig::MultivariateTrailStopValue = 0.0;

const double CConfig::PercentOfBalanceToTrade = 1.0;
const double CConfig::PercentOfTopCurrenciesToLearn = 1.0;
const int CConfig::SingleTradesAmount = 100;
const double CConfig::BiggerSlipPagePriceValue = 100.0;
const bool CConfig::DividedSets = true;
const int CConfig::NeuralNetworkLearnAttempts = 3;
const bool CConfig::UseOnlyCurrencyPairs = true;
IGenericObjectArray<IString> CConfig::Currencies;
IGenericObjectArray<IString> CConfig::Rules;
const int CConfig::LastHours = 6;

const bool CConfig::HeavyLoggingInFile = true;

const int CConfig::LogsAmountToAnalyze = 10;

const bool CConfig::ExecuteTrades = true;
const bool CConfig::ConsiderAccountCurrency = false;
const int CConfig::IntervalDoModifications = 5;
const int CConfig::IntervalDoLearning = 10 * 60;
const double CConfig::ProceedWithTradeLimit = 1.0;

const int CConfig::LotSizeForNoCurrencies = 5;

const double CConfig::PastPointsFraction = 2.0;
const int CConfig::NetworkInputsPointsAmount = 30;
const double CConfig::LearnParameter = 0.001;
const int CConfig::HiddenLayersAmount = 4;
const int CConfig::ActivationFunctionDimension = 3;
const int CConfig::HiddenLayersAmountForSymbolNetwork = 2;
const int CConfig::ActivationFunctionDimensionForSymbolNetwork = 3;
const int CConfig::ActivationFunctionDimensionForTrendNetwork = 3;
const int CConfig::HiddenLayersAmountForTrendNetwork = 3;
const int CConfig::MaxEpoch = 1000;
const int CConfig::MaxEpochSameValues = 100;
const double CConfig::ThresholdValue = 0.2;
const bool CConfig::UseMaxMultiTakeProfitStopLoss = true;
const double CConfig::TakeProfitPercentForStopLossToIgnore = 1.15;
const double CConfig::MultiPercent = 0.75;
const bool CConfig::MakeStopLossEqualTakeProfit = true;
const double CConfig::MinPMSValue = 0.001;
const int CConfig::MinTradableSymbols = 3;
const double CConfig::MinFirstPercentOfProfitReached = 0.2;
const double CConfig::MinSecondPercentOfProfitReached = 0.4;
const double CConfig::MinThirdPercentOfProfitReached = 0.6;
const double CConfig::MinFourthPercentOfProfitReached = 0.8;

const int CConfig::PastTradeMinAmount = 3;
const int CConfig::PastTradeMaxAmount = 20;
const int CConfig::PastTradeWithRTMaxAmount = 200;
const int CConfig::PastTradeWithRTMinAmount = 20;
const double CConfig::PastTradeWithRTSwitchRatio = 1.0;
const MethodDCombinationType CConfig::PastTradeWithRTCombination = MethodDCombinationType::BEST_COMMON;
const double CConfig::PastTradWithRTCommonCombinationIncludePercent = 0.25;
const int CConfig::PastTradeWithRTAutomaticParamsHoursMin = 6;
const int CConfig::PastTradeWithRTAutomaticParamsHoursMax = 36;
const int CConfig::PastTradeWithRTAutomaticParamsTradesMin = 5;
const int CConfig::PastTradeWithRTAutomaticParamsTradesMax = 20;
const double CConfig::PastTradeWithRTAutomaticParamsSwitchRatioMin = 0.5;
const double CConfig::PastTradeWithRTAutomaticParamsSwitchRatioMax = 1.5;
const double CConfig::PastTradeWithRTAutomaticParamsSwitchRatioStep = 0.1;
const bool CConfig::PastTradeWithRTAutomaticReverseTrade = true;

const double CConfig::PastTradeLineMultiplyMin = -2.0;
const double CConfig::PastTradeLineMultiplyMax = 2.0;
const int CConfig::PastTradeLineMultiplyStepsAmount = 100;
const int CConfig::PastTradesVectorsMaxAmount = 2000;
const bool CConfig::PastTrendsUseNeuralNetwork = true;

const double CConfig::GainsMinPercentage = 5.0;
const double CConfig::TradableOfLearnedPercent = 1.0;
const double CConfig::TestVectorsPercent = 0.2;
const double CConfig::TestPercentReachMin = 0.4;
const double CConfig::LearnPercentReach = 0.75;
const double CConfig::LearnPercentReachMin = 0.4;
const double CConfig::PercentPassedToAllPassedTest = 0.1;
const double CConfig::PercentPassedToAllPassedLearn = 0.25;

const double CConfig::TakeProfitPercentForStopLoss = 1.0; //For 1.0 StopLoss is max 100% of TakeProfit, 
														  //that means that statistically to make overall profit there is need for at least 50% of Trades to be Successfull.
const double CConfig::LastPeriodSuccessfullPercentMin = 0.5;

const double CConfig::PercentageOfChange = 0.003;
const double CConfig::TakeProfitStopLossBiggerSpan = 1.0;

const int CConfig::PastHoursCount = 10 * 24;
const int CConfig::PastHoursCountForAnalysis = 5 * 24;

//+------------------------------------------------------------------+
#endif
