/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_DISTANCES_PROVIDER
#define FILE_DISTANCES_PROVIDER

#include "../../Interoperability/GenericObjectArray.h"
#include "SymbolPointsList.h"
#include "CurrencyPairs.h"
#include "../../Interoperability/GenericArray.h"
#include "Math.h"
#include "../LibrariesInteroperability.h"
#include "../LibrariesLogger.h"

//+------------------------------------------------------------------+
class CDistancesProvider {
private:
	bool m_IsValid;
	CCurrencyPairs *m_Pairs;
	CValues *m_Values;
	int m_PointsAmount;

	IGenericObjectArray<CSymbolPointsList> m_PointsCurrBuy;
	IGenericObjectArray<CSymbolPointsList> m_PointsOtherBuy;
	IGenericObjectArray<CSymbolPointsList> m_PointsCurrSell;
	IGenericObjectArray<CSymbolPointsList> m_PointsOtherSell;

	IGenericObjectArray<IGenericArray<bool>> m_PointsCurrBuyLastMinimasMaximas;
	IGenericObjectArray<IGenericArray<bool>> m_PointsCurrSellLastMinimasMaximas;
	IGenericObjectArray<IGenericArray<bool>> m_PointsOtherBuyLastMinimasMaximas;
	IGenericObjectArray<IGenericArray<bool>> m_PointsOtherSellLastMinimasMaximas;

	IGenericObjectArray<IGenericArray<double>> m_PointsCurrBuyTimeLastMinimasMaximas;
	IGenericObjectArray<IGenericArray<double>> m_PointsCurrSellTimeLastMinimasMaximas;
	IGenericObjectArray<IGenericArray<double>> m_PointsOtherBuyTimeLastMinimasMaximas;
	IGenericObjectArray<IGenericArray<double>> m_PointsOtherSellTimeLastMinimasMaximas;

	void AssignValues(bool Symbol, bool Buy, int SymbolNr, int ParrelalSymbolNr, bool minimumMaximum, double timeDist)
	{
		if (Symbol)
		{
			if (Buy)
			{
				(*(m_PointsCurrBuyLastMinimasMaximas.GetPointerToValue(SymbolNr))).SetValue(ParrelalSymbolNr, minimumMaximum);
				(*(m_PointsCurrBuyTimeLastMinimasMaximas.GetPointerToValue(SymbolNr))).SetValue(ParrelalSymbolNr, timeDist);
			}
			else
			{
				(*(m_PointsCurrSellLastMinimasMaximas.GetPointerToValue(SymbolNr))).SetValue(ParrelalSymbolNr, minimumMaximum);
				(*(m_PointsCurrSellTimeLastMinimasMaximas.GetPointerToValue(SymbolNr))).SetValue(ParrelalSymbolNr, timeDist);
			}
		}
		else
		{
			if (Buy)
			{
				(*(m_PointsOtherBuyLastMinimasMaximas.GetPointerToValue(SymbolNr))).SetValue(ParrelalSymbolNr, minimumMaximum);
				(*(m_PointsOtherBuyTimeLastMinimasMaximas.GetPointerToValue(SymbolNr))).SetValue(ParrelalSymbolNr, timeDist);
			}
			else
			{
				(*(m_PointsOtherSellLastMinimasMaximas.GetPointerToValue(SymbolNr))).SetValue(ParrelalSymbolNr, minimumMaximum);
				(*(m_PointsOtherSellTimeLastMinimasMaximas.GetPointerToValue(SymbolNr))).SetValue(ParrelalSymbolNr, timeDist);
			}
		}
	}

	void InitLastSizes(int size)
	{
		m_PointsAmount = size;
		m_PointsCurrBuyLastMinimasMaximas.Resize(size);
		for (int i = 0; i < size; i++)
		{
			(*(m_PointsCurrBuyLastMinimasMaximas.GetPointerToValue(i))).Resize(m_PointsCurrBuy.Size());
		}
		m_PointsCurrSellLastMinimasMaximas.Resize(size);
		for (int i = 0; i < size; i++)
		{
			(*(m_PointsCurrSellLastMinimasMaximas.GetPointerToValue(i))).Resize(m_PointsCurrSell.Size());
		}
		m_PointsOtherBuyLastMinimasMaximas.Resize(size);
		for (int i = 0; i < size; i++)
		{
			(*(m_PointsOtherBuyLastMinimasMaximas.GetPointerToValue(i))).Resize(m_PointsOtherBuy.Size());
		}
		m_PointsOtherSellLastMinimasMaximas.Resize(size);
		for (int i = 0; i < size; i++)
		{
			(*(m_PointsOtherSellLastMinimasMaximas.GetPointerToValue(i))).Resize(m_PointsOtherSell.Size());
		}

		m_PointsCurrBuyTimeLastMinimasMaximas.Resize(size);
		for (int i = 0; i < size; i++)
		{
			(*(m_PointsCurrBuyTimeLastMinimasMaximas.GetPointerToValue(i))).Resize(m_PointsCurrBuy.Size());
		}
		m_PointsCurrSellTimeLastMinimasMaximas.Resize(size);
		for (int i = 0; i < size; i++)
		{
			(*(m_PointsCurrSellTimeLastMinimasMaximas.GetPointerToValue(i))).Resize(m_PointsCurrSell.Size());
		}
		m_PointsOtherBuyTimeLastMinimasMaximas.Resize(size);
		for (int i = 0; i < size; i++)
		{
			(*(m_PointsOtherBuyTimeLastMinimasMaximas.GetPointerToValue(i))).Resize(m_PointsOtherBuy.Size());
		}
		m_PointsOtherSellTimeLastMinimasMaximas.Resize(size);
		for (int i = 0; i < size; i++)
		{
			(*(m_PointsOtherSellTimeLastMinimasMaximas.GetPointerToValue(i))).Resize(m_PointsOtherSell.Size());
		}
	}

	CSymbolPointsList* ReturnSymbolPointsList(bool Symbol, bool Buy, int ListNr)
	{
		if (Symbol)
		{
			if (Buy)
			{
				return m_PointsCurrBuy.GetPointerToValue(ListNr);
			}
			else
			{
				return m_PointsCurrSell.GetPointerToValue(ListNr);
			}
		}
		else
		{
			if (Buy)
			{
				return m_PointsOtherBuy.GetPointerToValue(ListNr);
			}
			else
			{
				return m_PointsOtherSell.GetPointerToValue(ListNr);
			}
		}
	}

	void Correlations(IGenericArray<bool> &minimasMaximas, IGenericArray<double> &timeDistances, IGenericArray<bool> &otherMinimasMaximas, IGenericArray<double> &otherTimeDistances, double &positiveCorrelations, double &negativeCorrelations)
	{
		double val;
		double otherVal;
		
		for (int i = 0; i < minimasMaximas.Size(); i++)
		{
			val = timeDistances.GetValue(i);
			otherVal = otherTimeDistances.GetValue(i);

			if (val >= 0.0 && otherVal >= 0)
			{
				if (minimasMaximas.GetValue(i) == otherMinimasMaximas.GetValue(i))
				{
					positiveCorrelations += GetInteroperability().MathAbs(val - otherVal);
				}
				else
				{
					negativeCorrelations += GetInteroperability().MathAbs(val - otherVal);
				}
			}
		}
	}

	void CalculateCorrelations(double &positiveCorrelations, double &negativeCorrelations)
	{
		positiveCorrelations = 0.0;
		negativeCorrelations = 0.0;

		for (int i = 0; i < m_PointsAmount; i++)
		{
    		Correlations(CurrBuyMinimasMaximas, CurrBuyTimes, (*(m_PointsCurrBuyLastMinimasMaximas.GetPointerToValue(i))), (*(m_PointsCurrBuyTimeLastMinimasMaximas.GetPointerToValue(i))), positiveCorrelations, negativeCorrelations);
			Correlations(CurrSellMinimasMaximas, CurrSellTimes, (*(m_PointsCurrSellLastMinimasMaximas.GetPointerToValue(i))), (*(m_PointsCurrSellTimeLastMinimasMaximas.GetPointerToValue(i))), positiveCorrelations, negativeCorrelations);
			Correlations(OtherBuyMinimasMaximas, OtherBuyTimes, (*(m_PointsOtherBuyLastMinimasMaximas.GetPointerToValue(i))), (*(m_PointsOtherBuyTimeLastMinimasMaximas.GetPointerToValue(i))), positiveCorrelations, negativeCorrelations);
			Correlations(OtherSellMinimasMaximas, OtherSellTimes, (*(m_PointsOtherSellLastMinimasMaximas.GetPointerToValue(i))), (*(m_PointsOtherSellTimeLastMinimasMaximas.GetPointerToValue(i))), positiveCorrelations, negativeCorrelations);
		}
	}

	void InitCurrent()
	{
		CurrBuyMinimasMaximas.Resize((*m_Pairs).AmountCurrencyPairsToBuy());
		CurrSellMinimasMaximas.Resize((*m_Pairs).AmountCurrencyPairsToSell());
		OtherBuyMinimasMaximas.Resize((*m_Pairs).AmountOtherSymbolsToBuy());
		OtherSellMinimasMaximas.Resize((*m_Pairs).AmountOtherSymbolsToSell());

		CurrBuyTimes.Resize((*m_Pairs).AmountCurrencyPairsToBuy());
		CurrSellTimes.Resize((*m_Pairs).AmountCurrencyPairsToSell());
		OtherBuyTimes.Resize((*m_Pairs).AmountOtherSymbolsToBuy());
		OtherSellTimes.Resize((*m_Pairs).AmountOtherSymbolsToSell());
	}

public:
	IGenericArray<bool> CurrBuyMinimasMaximas;
	IGenericArray<bool> CurrSellMinimasMaximas;
	IGenericArray<bool> OtherBuyMinimasMaximas;
	IGenericArray<bool> OtherSellMinimasMaximas;

	IGenericArray<double> CurrBuyTimes;
	IGenericArray<double> CurrSellTimes;
	IGenericArray<double> OtherBuyTimes;
	IGenericArray<double> OtherSellTimes;

	void SetValidity(bool valid)
	{
		m_IsValid = valid;
	}

	bool Calculate(bool Symbol, bool Buy, int SymbolNr, bool &ReverseTrades)
	{
		CSymbolPointsList *symbolsList;
		CSymbolPointsList *mainSymbolList;
		IDateTime date;
		CPastPoint *point;
		int pointNr;

		mainSymbolList = ReturnSymbolPointsList(Symbol, Buy, SymbolNr);
		InitLastSizes((*mainSymbolList).PastPointsAmount());

		for (int pos = 0; pos < (*mainSymbolList).PastPointsAmount(); pos++)
		{
			date = (*((*mainSymbolList).GetPastPoint(pos))).DateOfMinimumMaximum;
			for (int nr = 0; nr < (*mainSymbolList).PastPointsAmount(); nr++)
			{
				for (int i = 0; i < (*m_Pairs).AmountCurrencyPairsToBuy(); i++)
				{
					symbolsList = ReturnSymbolPointsList(true, true, i);
					pointNr = (*symbolsList).GetFirstPastPointBefore(date);
					if (pointNr >= 0)
					{
						point = (*symbolsList).GetPastPoint(pointNr);
						AssignValues(Symbol, Buy, nr, i, (*point).WasMaximum, IDateTime::SecondsDiff((*point).DateOfMinimumMaximum, date));
					}
					else
					{
						AssignValues(Symbol, Buy, nr, i, false, -1.0);
					}
				}

				for (int i = 0; i < (*m_Pairs).AmountCurrencyPairsToSell(); i++)
				{
					symbolsList = ReturnSymbolPointsList(true, false, i);
					pointNr = (*symbolsList).GetFirstPastPointBefore(date);
					if (pointNr >= 0)
					{
						point = (*symbolsList).GetPastPoint(pointNr);
						AssignValues(Symbol, Buy, nr, i, (*point).WasMaximum, IDateTime::SecondsDiff((*point).DateOfMinimumMaximum, date));
					}
					else
					{
						AssignValues(Symbol, Buy, nr, i, false, -1.0);
					}
				}

				for (int i = 0; i < (*m_Pairs).AmountOtherSymbolsToBuy(); i++)
				{
					symbolsList = ReturnSymbolPointsList(false, true, i);
					pointNr = (*symbolsList).GetFirstPastPointBefore(date);
					if (pointNr >= 0)
					{
						point = (*symbolsList).GetPastPoint(pointNr);
						AssignValues(Symbol, Buy, nr, i, (*point).WasMaximum, IDateTime::SecondsDiff((*point).DateOfMinimumMaximum, date));
					}
					else
					{
						AssignValues(Symbol, Buy, nr, i, false, -1.0);
					}
				}

				for (int i = 0; i < (*m_Pairs).AmountOtherSymbolsToSell(); i++)
				{
					symbolsList = ReturnSymbolPointsList(false, false, i);
					pointNr = (*symbolsList).GetFirstPastPointBefore(date);
					if (pointNr >= 0)
					{
						point = (*symbolsList).GetPastPoint(pointNr);
						AssignValues(Symbol, Buy, nr, i, (*point).WasMaximum, IDateTime::SecondsDiff((*point).DateOfMinimumMaximum, date));
					}
					else
					{
						AssignValues(Symbol, Buy, nr, i, false, -1.0);
					}
				}
			}
		}

		double positiveCorrelations;
		double negativeCorrelations;
		int Amount;

		CalculateCorrelations(positiveCorrelations, negativeCorrelations);
		Amount = (*m_Pairs).AmountCurrencyPairsToBuy() + (*m_Pairs).AmountCurrencyPairsToSell() + (*m_Pairs).AmountOtherSymbolsToBuy() + (*m_Pairs).AmountOtherSymbolsToSell();

		if (Amount > 0)
		{
			positiveCorrelations /= Amount;
			negativeCorrelations /= Amount;
		}

		if (positiveCorrelations == 0.0 && negativeCorrelations == 0.0)
		{
			return false;
		}

		double pos = positiveCorrelations;

		positiveCorrelations /= (GetInteroperability().MathAbs(pos) + GetInteroperability().MathAbs(negativeCorrelations));
		negativeCorrelations /= (GetInteroperability().MathAbs(pos) + GetInteroperability().MathAbs(negativeCorrelations));

		double percent = 0.0;

		if(positiveCorrelations != 0.0 && negativeCorrelations != 0.0)
		{
			percent = CMath::Smaller(positiveCorrelations, negativeCorrelations) / CMath::Bigger(positiveCorrelations, negativeCorrelations);
			percent = 1.0 - percent;
			percent *= 100.0;
		}		

		GetLogger().LogSpecial(IString("Positive, Negative Correlations : ") + GetInteroperability().DoubleToString(1.0 - positiveCorrelations) + IString(", ") + GetInteroperability().DoubleToString(1.0 - negativeCorrelations) + IString(", Difference : ") + GetInteroperability().DoubleToString(positiveCorrelations - negativeCorrelations) + IString(", ") + GetInteroperability().DoubleToString(percent) + IString("%"));

		if (percent >= CConfig::MinPercentageDistanceDifference)
		{
			if ((1.0 - positiveCorrelations) > (1.0 - negativeCorrelations))
			{
				ReverseTrades = false;
				GetLogger().LogSpecial(IString("Method F, ReverseTrades : ") + GetInteroperability().BoolToString(false));			
				if (CConfig::ConsiderPositiveCorrelation)
				{
					return true;
				}
			}
			else
			{
				ReverseTrades = true;
				GetLogger().LogSpecial(IString("Method F, ReverseTrades : ") + GetInteroperability().BoolToString(true));
				if (CConfig::ConsiderNegativeCorrelation)
				{
					return true;
				}
			}			
		}
		return false;
	}

	bool IsValid()
	{
		return m_IsValid;
	}

	void AssignPoints(IGenericObjectArray<CPastPoint> &pastPointsBuys, IGenericObjectArray<CPastPoint> &pastPointsSells, CCurrencyPairs &pairs, CValues &values)
	{
		m_Values = &values;
		m_Pairs = &pairs;
		m_PointsCurrBuy.Resize((*m_Pairs).AmountCurrencyPairsToBuy());
		m_PointsOtherBuy.Resize((*m_Pairs).AmountOtherSymbolsToBuy());
		for (int i = 0; i < pastPointsBuys.Size(); i++)
		{
			if ((*(pastPointsBuys.GetPointerToValue(i))).Symbol)
			{
				(*(m_PointsCurrBuy.GetPointerToValue((*(pastPointsBuys.GetPointerToValue(i))).SymbolNr))).AddPastPoint((*(pastPointsBuys.GetPointerToValue(i))));
			}
			else
			{
				(*(m_PointsOtherBuy.GetPointerToValue((*(pastPointsBuys.GetPointerToValue(i))).SymbolNr))).AddPastPoint((*(pastPointsBuys.GetPointerToValue(i))));
			}
		}
		m_PointsCurrSell.Resize((*m_Pairs).AmountCurrencyPairsToSell());
		m_PointsOtherSell.Resize((*m_Pairs).AmountOtherSymbolsToSell());
		for (int i = 0; i < pastPointsSells.Size(); i++)
		{
			if ((*(pastPointsSells.GetPointerToValue(i))).Symbol)
			{
				(*(m_PointsCurrSell.GetPointerToValue((*(pastPointsSells.GetPointerToValue(i))).SymbolNr))).AddPastPoint((*(pastPointsSells.GetPointerToValue(i))));
			}
			else
			{
				(*(m_PointsOtherSell.GetPointerToValue((*(pastPointsSells.GetPointerToValue(i))).SymbolNr))).AddPastPoint((*(pastPointsSells.GetPointerToValue(i))));
			}
		}
		for (int i = 0; i < m_PointsCurrBuy.Size(); i++)
		{
			(*(m_PointsCurrBuy.GetPointerToValue(i))).InitSort();
		}
		for (int i = 0; i < m_PointsCurrSell.Size(); i++)
		{
			(*(m_PointsCurrSell.GetPointerToValue(i))).InitSort();
		}
		for (int i = 0; i < m_PointsOtherBuy.Size(); i++)
		{
			(*(m_PointsOtherBuy.GetPointerToValue(i))).InitSort();
		}
		for (int i = 0; i < m_PointsOtherSell.Size(); i++)
		{
			(*(m_PointsOtherSell.GetPointerToValue(i))).InitSort();
		}
		InitCurrent();
	}
};
//+------------------------------------------------------------------+
#endif