/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_FAR_PAST_DATA_FILE
#define FILE_FAR_PAST_DATA_FILE

#include "../../Interoperability/FileWriter.h"
#include "../../Interoperability/FileReader.h"
#include "CurrencyPairs.h"
#include "Values.h"
#include "Config.h"

//+------------------------------------------------------------------+
//Writing an Reading Past Symbols Values from file system
class CFarPastDataFile {
private:
	static IFileWriter Writer;//File that Symbols Values will be writen to

	static bool Parse(IString &line, CCurrencyPairs &pairs, CPastValues &values)//Parse line from the file and extract Date and Values of the Symbols
	{
		IGenericObjectArray<IString> arr;

		line.StringSplit(';', arr);
		if (GetInteroperability().StringToDouble((*(arr.GetPointerToValue(1)))) != 0.0)
		{
			values.Date = GetInteroperability().StringToTime((*(arr.GetPointerToValue(0))));
			values.Values.Resize(arr.Size() - 2);
			for (int i = 1; i < arr.Size() - 1 ; i++)
			{
				values.Values.SetValue(i - 1, GetInteroperability().StringToDouble((*(arr.GetPointerToValue(i)))));
			}
			return true;
		}
		return false;
	}

	static int CountLines(IFileReader &reader)
	{
		int am = 0;
		IString line;

		line = reader.ReadLine();
		while (!IString::AreEqual(line, IString("")))
		{
			am++;
			line = reader.ReadLine();
		}
		reader.SeekBeg();
		return am;
	}

public:
	static void Init()//Init file that will holdpast values of Symbols
	{
		IString name("FarPastData.txt");

		Writer.Initialize(name);
	}

	static void WriteDataColumnsValues(CCurrencyPairs &pairs)//Write to file Date and Values of all Present Symbols
	{
		IString dataNames(""), name;
		IDateTime time = IDateTime::TimeCurrent();

		dataNames += (time.TimeCurrentAsString() + IString(";"));
		for (int i = 0; i < pairs.AmountCurrencyPairsToBuy(); i++)
		{
			name = pairs.ReturnCurrencyPairToBuyNr(i);
			dataNames += (GetInteroperability().DoubleToString(CValues::BuyPrice(name)) + IString(";"));
		}
		for (int i = 0; i < pairs.AmountCurrencyPairsToSell(); i++)
		{
			name = pairs.ReturnCurrencyPairToSellNr(i);
			dataNames += (GetInteroperability().DoubleToString(CValues::BuyPrice(name)) + IString(";"));
		}
		for (int i = 0; i < pairs.AmountOtherSymbolsToBuy(); i++)
		{
			name = pairs.ReturnOtherSymbolToBuyNr(i);
			dataNames += (GetInteroperability().DoubleToString(CValues::BuyPrice(name)) + IString(";"));
		}
		for (int i = 0; i < pairs.AmountOtherSymbolsToSell(); i++)
		{
			name = pairs.ReturnOtherSymbolToSellNr(i);
			dataNames += (GetInteroperability().DoubleToString(CValues::BuyPrice(name)) + IString(";"));
		}
		Writer.Write(dataNames);
	}

    static void WriteDataColumnsNames(CCurrencyPairs &pairs)//Write to file Names of all Symbols
	{
		IString dataNames("");
		IDateTime time = IDateTime::TimeCurrent();

		dataNames += (time.TimeCurrentAsString() + IString(";"));
		for (int i = 0; i < pairs.AmountCurrencyPairsToBuy(); i++)
		{
			dataNames += (pairs.ReturnCurrencyPairToBuyNr(i) + IString(";"));
		}
		for (int i = 0; i < pairs.AmountCurrencyPairsToSell(); i++)
		{
			dataNames += (pairs.ReturnCurrencyPairToSellNr(i) + IString(";"));
		}
		for (int i = 0; i < pairs.AmountOtherSymbolsToBuy(); i++)
		{
			dataNames += (pairs.ReturnOtherSymbolToBuyNr(i) + IString(";"));
		}
		for (int i = 0; i < pairs.AmountOtherSymbolsToSell(); i++)
		{
			dataNames += (pairs.ReturnOtherSymbolToSellNr(i) + IString(";"));
		}
		Writer.Write(dataNames);
	}

	static void ReadDataColumnsValues(CCurrencyPairs &pairs, IGenericObjectArray<CPastValues> &pastValues)//Read Past Values with Dates from file
	{
		IString name("FarPastData.txt");
		IFileReader Reader;
		IString line;
		CPastValues values;
		double lines;
		double pos = 0.0;

		Reader.Initialize(name);
		lines = CountLines(Reader);
		
		if (lines > CConfig::MaxPastDataLength * 24 * 60)
		{
			pos = (lines - CConfig::MaxPastDataLength * 24 * 60) / lines;
			Reader.Seek(pos);
		}
		
		line = Reader.ReadLine();
		line = Reader.ReadLine();
		while (!IString::AreEqual(line, IString("")))
		{
			if (Parse(line, pairs, values))
			{
				pastValues.GrowIfNeeded();
				CPastValues::CopyValues(values, (*(pastValues.GetPointerToValue(pastValues.LastNr()))));
				pastValues.IncreaseSize();
			}
			line = Reader.ReadLine();
		}
	}

	static bool IsPastDataLongEnough(IGenericObjectArray<CPastValues> &pastValues)//Checks whenever Past Dates Amount is bigger than specified Constant
	{
		if (pastValues.Size() > 0 && IDateTime::MinutesDiff((*(pastValues.GetPointerToValue(0))).Date, (*(pastValues.GetPointerToValue(pastValues.Size() - 1))).Date) > CConfig::MinPastDataLength * 24 * 60)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
};
IFileWriter CFarPastDataFile::Writer;
//+------------------------------------------------------------------+
#endif