/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_LOG_PROGRESS
#define FILE_LOG_PROGRESS

#include "../../Interoperability/String.h"
#include "../LibrariesInteroperability.h"
#include "../LibrariesLogger.h"

//+------------------------------------------------------------------+
//Generating Progress message
class CLogProgress {
private:
	double AMOUNT;
	double m_Max;//Max progress to be displayed
	int m_LastDisplayed;//Last value of progress displayed
	IString m_Msg;//Message prefix

public:
	CLogProgress()
	{
		AMOUNT = 10.0;
		m_LastDisplayed = 0;
	}

	void Init(IString &msg, int max)//Initializing Message prefix and max value of progress
	{
		m_Max = max;
		m_Msg = msg;
	}

	void Log(int nr)//Log value of Progress
	{
		double display = nr * AMOUNT / m_Max;

		if (m_LastDisplayed < (int)(display))
		{
			GetLogger().LogSpecial(m_Msg + GetInteroperability().DoubleToString(display));
			m_LastDisplayed = (int)(display);
		}
	}
};
//+------------------------------------------------------------------+
#endif