/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_LOGGER
#define FILE_LOGGER

#include "Config.h"
#include "../../Interoperability/String.h"
#include "../../Interoperability/FileWriter.h"
#include "../LibrariesInteroperability.h"
#include "../../Interoperability/DateTime.h"

//+------------------------------------------------------------------+
//Used to post logs from all aplication into file system
class CLogger {
private:
	static IFileWriter Writer;//File representing log that is most heavy with logging information
	static IFileWriter WriterSpecial;//File representing log that gets displayd at runtime and is put to file special
	static IFileWriter WriterTrades;//File representing log with opened Trades
	static IFileWriter WriterModificators;//File representing log with information from Open Trades Modificators
	static IFileWriter WriterProfitPast;//File representing log of profit values in time
	static IFileWriter WriterSpeeds;//File representing log of speeds
	static bool Initialized;

public:
	void static Init(IString &name)//Initialization of all log files
	{
		if (!Initialized)
		{
			IString time = IDateTime::TimeCurrentAsString();

			time.StringReplace(".", "_");
			time.StringReplace(":", "_");
			time.StringReplace(" ", "_");

			IString nameW;

			nameW.AssignString(name + "_" + time + ".txt");
			Writer.Initialize(nameW);

			nameW.AssignString(IString("special") + "_" + time + ".txt");
			WriterSpecial.Initialize(nameW);

			nameW.AssignString(IString("trades") + "_" + time + ".txt");
			WriterTrades.Initialize(nameW);

			nameW.AssignString(IString("modificators") + "_" + time + ".txt");
			WriterModificators.Initialize(nameW);

			nameW.AssignString(IString("ProfitPastValues.txt"));
			WriterProfitPast.Initialize(nameW);

			nameW.AssignString(IString("speeds") + "_" + time + ".txt");
			WriterSpeeds.Initialize(nameW);

			Initialized = true;
		}
	}

	void Log(const IString &msg)//Displaying information at runtime
	{
	   IString tmp(msg);
	
		GetInteroperability().Print(tmp);
		LogInFile(tmp);
	}

	void LogInFile(const IString &msg)//Logging the information to file with most heavy logging
	{
		IString time(IDateTime::TimeCurrentAsString());
      IString tmp(msg);
       
		Writer.Write(time + ", " + tmp);
	}

	void HeavyLog(const IString &msg)//Logging the information to file with most heavy logging while checking if HeavyLogging is enabled
	{
		if (CConfig::HeavyLoggingInFile)
		{
			LogInFile(msg);
		}
	}

	void LogSpecial(const IString &msg)//Logging to special log, display, and heavy log
	{
		IString time(IDateTime::TimeCurrentAsString());
      IString tmp(msg);

		WriterSpecial.Write(time + ", " + tmp);
		Log(tmp);
	}

	void LogSpeeds(const IString &msg)//Logging to speeds log
	{
		IString time(IDateTime::TimeCurrentAsString());
		IString tmp(msg);

		WriterSpeeds.Write(time + ", " + tmp);
	}

	void LogTrade(const IString &msg)//Logging to Trade log, display, heavy log
	{
		IString time(IDateTime::TimeCurrentAsString());
      IString tmp(msg);
      
		WriterTrades.Write(time + ", " + tmp);
		Log(tmp);
	}

	void LogFromModificator(const IString &msg)//Logging to Modificators log, display, heavy log
	{
		IString time;
      IString tmp(msg);

		time = IDateTime::TimeCurrentAsString();

		WriterModificators.Write(time + ", " + tmp);
		Log(tmp);
	}

	void LogFromProfitPast(const IString &msg)//Logging to Profits Log
	{
		IString time;
      IString tmp(msg);
      
		time = IDateTime::TimeCurrentAsString();

		WriterProfitPast.Write(time + ", " + tmp);
	}
};
IFileWriter CLogger::Writer;
IFileWriter CLogger::WriterSpecial;
IFileWriter CLogger::WriterTrades;
IFileWriter CLogger::WriterModificators;
IFileWriter CLogger::WriterProfitPast;
IFileWriter CLogger::WriterSpeeds;
bool CLogger::Initialized = false;
#endif