/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_OPEN_TRADES
#define FILE_OPEN_TRADES

#include "../../Interoperability/SortableArray.h"
#include "../../Interoperability/GenericArray.h"

//+------------------------------------------------------------------+
class COpenTrades{
    private:
		ISortableArray<int> m_OpenTrades;
		ISortableArray<int> m_OpenReverseTrades;

		void RemoveOpenTrade(int tradeNr)
		{
			int pos = m_OpenTrades.BinarySearch(tradeNr);

			if (m_OpenTrades.Size() > 0 && m_OpenTrades.GetValue(pos) == tradeNr)
			{
				m_OpenTrades.RemoveNr(pos);
			}
		}

		void RemoveReverseOpenTrade(int tradeNr)
		{
			int pos = m_OpenReverseTrades.BinarySearch(tradeNr);

			if (m_OpenReverseTrades.Size() > 0 && m_OpenReverseTrades.GetValue(pos) == tradeNr)
			{
				m_OpenReverseTrades.RemoveNr(pos);
			}
		}

    public:
		void AddOpenTrade(int tradeNr)
		{
			if (m_OpenTrades.Size() > 0)
			{
				int pos;
				pos = m_OpenTrades.BinarySearch(tradeNr);

				if (m_OpenTrades.GetValue(pos) == tradeNr)
				{
					return;
				}
			}
			m_OpenTrades.Resize(m_OpenTrades.Size() + 1);
			m_OpenTrades.SetValue(m_OpenTrades.Size() - 1, tradeNr);
			m_OpenTrades.SetPos(m_OpenTrades.Size() - 1, tradeNr);
			m_OpenTrades.Sort();
     	}

		void AddReverseOpenTrade(int tradeNr)
		{
			if (m_OpenReverseTrades.Size() > 0)
			{
				int pos;
				pos = m_OpenReverseTrades.BinarySearch(tradeNr);

				if (m_OpenReverseTrades.GetValue(pos) == tradeNr)
				{
					return;
				}
			}
			m_OpenReverseTrades.Resize(m_OpenReverseTrades.Size() + 1);
			m_OpenReverseTrades.SetValue(m_OpenReverseTrades.Size() - 1, tradeNr);
			m_OpenReverseTrades.SetPos(m_OpenReverseTrades.Size() - 1, tradeNr);
			m_OpenReverseTrades.Sort();
		}

		void RemoveNotPresent(IGenericArray<int> &numbers)
		{
			ISortableArray<int> sorted;
			int nr;

			sorted.Resize(numbers.Size());
			for (int i = 0; i < sorted.Size(); i++)
			{
				sorted.SetValue(i, numbers.GetValue(i));
				sorted.SetPos(i, numbers.GetValue(i));
			}
			sorted.Sort();
			for (int i = 0; i < m_OpenTrades.Size(); i++)
			{
				nr = sorted.BinarySearch(m_OpenTrades.GetPos(i));
				if (m_OpenTrades.GetPos(i) != sorted.GetPos(nr))
				{
					m_OpenTrades.RemoveNr(i);
				}
			}
			for (int i = 0; i < m_OpenReverseTrades.Size(); i++)
			{
				nr = sorted.BinarySearch(m_OpenReverseTrades.GetPos(i));
				if (m_OpenReverseTrades.GetPos(i) != sorted.GetPos(nr))
				{
					m_OpenReverseTrades.RemoveNr(i);
				}
			}
		}

		bool IsReverseTrade(int tradeNr)
		{
			int nr = m_OpenReverseTrades.BinarySearch(tradeNr);

			if (nr >= 0)
			{
				return (m_OpenReverseTrades.GetPos(nr) == tradeNr);
			}
			else
			{
				return false;
			}
		}

		ISortableArray<int>* ReturnOpenTrades()
		{
			return &m_OpenTrades;
		}

		ISortableArray<int>* ReturnReverseOpenTrades()
		{
			return &m_OpenReverseTrades;
		}
};
//+------------------------------------------------------------------+
#endif