/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_PAST_POINT
#define FILE_PAST_POINT

#include "Config.h"
#include "../../Interoperability/DateTime.h"

//+------------------------------------------------------------------+
class CPastPoint{
    public:
        int SymbolNr;
        int MinimumMaximumPointNr;
        int EstablishingMinimumMaximumPointNr;
        bool Successful;
		bool WasMaximum;

        bool Symbol;
        bool SymbolBuy;
        
        double Val;
        double Diff;
        IDateTime ValDate;
        
        double PossibleLoss;
        double PossibleProfit;
        
		IDateTime DateOfMinimumMaximum;

        bool IsValid()
        {
            return PastPointsBegin() >= 0;
        }
        
        bool IsIn(double lossMax, double profitMin)
        {
            if(lossMax >= PossibleLoss && profitMin <= PossibleProfit)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        
        int PastPointsBegin()
        {
            int nr = EstablishingMinimumMaximumPointNr - MinimumMaximumPointNr;
            
            return (int)(MinimumMaximumPointNr - nr * CConfig::PastPointsFraction);
        }
        
        CPastPoint()
        { }
        
        CPastPoint(const CPastPoint &other)
        {
            SymbolNr = other.SymbolNr;
            MinimumMaximumPointNr = other.MinimumMaximumPointNr;
            EstablishingMinimumMaximumPointNr = other.EstablishingMinimumMaximumPointNr;
            Successful = other.Successful;
            Symbol = other.Symbol;
            SymbolBuy = other.SymbolBuy;
			DateOfMinimumMaximum = other.DateOfMinimumMaximum;
			WasMaximum = other.WasMaximum;
        }
        
        int PointsAmount()
        {
            return EstablishingMinimumMaximumPointNr - PastPointsBegin();
        }
        
        int ReturnPointNr(int nr)
        {
            return PastPointsBegin() + nr;
        }
};
//+------------------------------------------------------------------+
#endif