/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_PAST_PROFITS
#define FILE_PAST_PROFITS

#include "Config.h"
#include "../../Interoperability/DateTime.h"
#include "../../Interoperability/GenericArray.h"
#include "../../Interoperability/GenericObjectArray.h"

//+------------------------------------------------------------------+
class CPastProfits{
    private:
	    IGenericArray<double> m_Profits;
	    IGenericObjectArray<IDateTime> m_Dates;

		void Move()
		{
			int i;

			for (i = 1; i < m_Profits.Size(); i++)
			{
				m_Profits.SetValue(i - 1, m_Profits.GetValue(i));
				(*(m_Dates.GetPointerToValue(i - 1))).Assign((*(m_Dates.GetPointerToValue(i))));
			}
			m_Profits.Remove(i - 1);
			m_Dates.Remove(i - 1);
		}
    public:
		void AddProfit(double profit, IDateTime &date, double hours)
		{
			if (m_Profits.Size() > 1)
			{
				while (IDateTime::HoursDiffWithoutWeekends((*(m_Dates.GetPointerToValue(0))), date) > hours && IDateTime::HoursDiffWithoutWeekends((*(m_Dates.GetPointerToValue(1))), date) > hours)
				{
					Move();
					if (m_Profits.Size() == 0)
					{
						break;
					}
				}
			}
			m_Profits.Add(profit);
			m_Dates.Add(date);
		}

		bool IsValid(double hours)
		{
    		if (m_Dates.Size() > 1)
			{
				if (IDateTime::HoursDiffWithoutWeekends((*(m_Dates.GetPointerToValue(0))), (*(m_Dates.GetPointerToValue(m_Dates.Size() - 1)))) > 0.9 * hours)
				{
					return true;
				}
			}
			return false;
		}

		bool IsSpeedValid(double hours)
		{
			if (m_Dates.Size() > 1)
			{
				if (IDateTime::HoursDiffWithoutWeekends((*(m_Dates.GetPointerToValue(0))), (*(m_Dates.GetPointerToValue(m_Dates.Size() - 1)))) >= hours)
				{
					return true;
				}
			}
			return false;
		}

		double Speed(double hours)
		{
			return (m_Profits.GetValue(m_Dates.Size() - 1) - m_Profits.GetValue(0)) / IDateTime::HoursDiffWithoutWeekends((*(m_Dates.GetPointerToValue(0))), (*(m_Dates.GetPointerToValue(m_Dates.Size() - 1))));
		}

		double PastProfit()
		{
			if (m_Profits.Size() > 0)
			{
				return m_Profits.GetValue(0);
			}
			else
			{
				return 0.0;
			}
		}

		void EmptyProfits()
		{
			m_Profits.Resize(0);
			m_Dates.Resize(0);
		}
};
//+------------------------------------------------------------------+
#endif