/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_SYMBOL_POINTS_LIST
#define FILE_SYMBOL_POINTS_LIST

#include "../../Interoperability/GenericObjectArray.h"
#include "PastPoint.h"
#include "../../Interoperability/SortableArray.h"

//+------------------------------------------------------------------+
class CSymbolPointsList{
    private:
		IGenericObjectArray<CPastPoint> m_Points;
		ISortableArray<double> m_Sortable;

    public:
		void AddPastPoint(CPastPoint &point)
		{
			m_Points.GrowIfNeeded();
			(*(m_Points.GetPointerToValue(m_Points.LastNr()))) = point;
			m_Points.IncreaseSize();
		}

		CPastPoint* GetPastPoint(int nr)
		{
			return m_Points.GetPointerToValue(nr);
		}

		int PastPointsAmount()
		{
			return m_Points.Size();
		}

		void InitSort()
		{
			m_Sortable.Resize(m_Points.Size());
			for (int i = 0; i < m_Sortable.Size(); i++)
			{
				m_Sortable.SetPos(i, i);
				m_Sortable.SetValue(i, (double)((*(m_Points.GetPointerToValue(i))).Val));
			}
			m_Sortable.Sort();
		}

		int GetFirstPastPointBefore(IDateTime &date)
		{
			int pos = m_Sortable.BinarySearch((double)date.GetPlatformTime());

			if (pos > 0)
			{
				if (date.GetPlatformTime() >= m_Sortable.GetValue(pos))
				{
					return pos;
				}
				else
				{
					return pos - 1;
				}
			}
			else if (pos == 0)
			{
				return pos;
			}
			else
			{
				return -1;
			}
		}
};
//+------------------------------------------------------------------+
#endif