/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_TRADING
#define FILE_TRADING

#include "../LibrariesInteroperability.h"
#include "../Librarieslogger.h"
#include "Values.h"
#include "Config.h"
#include "CurrencyPairs.h"
#include "../../Interoperability/DateTime.h" 
#include "Varibles.h"
#include "Fcnnhn/Rand.h"

//+------------------------------------------------------------------+
class CTrading{
    private:
		static CVaribles m_Varibles;
		        
		static bool Proceed()
		{
			if (CRand::DoubleFromRange(0.0, 1.0) <= CConfig::ProceedWithTradeLimit)
			{
				return true;
			}
			else
			{
				return false;
			}
		}
                
        static bool IsBiggerSlipPage(double price)
        {
            return price >= CConfig::BiggerSlipPagePriceValue;
        }
        
        static void Log(IString &symbol)
        {
            IDateTime now = IDateTime::TimeCurrent();

            GetLogger().LogSpecial(IString("Log Symbol : ") + symbol + ", iHigh : " + GetInteroperability().DoubleToString(CValues::GetPriceAtTimeHigh(symbol, now)) + ", iLow : " + GetInteroperability().DoubleToString(CValues::GetPriceAtTime(symbol,now)) + ", Sell : " + GetInteroperability().DoubleToString(CValues::SellPrice(symbol)) + ", Buy : " + GetInteroperability().DoubleToString(CValues::BuyPrice(symbol)) + ", Spread : " + GetInteroperability().DoubleToString(CValues::Spread(symbol)));
        }
        
		unsigned long m_Ticket;
		int m_TicketNr;

    public:
        static int SendOrder(IString& symbol, double price, bool buy, double takeProfit, double stopLoss, double lotSize)
        {
            IString type;

            price = GetInteroperability().NormalizeDouble(price, GetInteroperability().Digits());
            takeProfit = GetInteroperability().NormalizeDouble(takeProfit, GetInteroperability().Digits());
            stopLoss = GetInteroperability().NormalizeDouble(stopLoss, GetInteroperability().Digits());

            if (buy)
            {
                type.AssignString("Buy");
            }
            else
            {
                type.AssignString("Sell");
            }

            int nr = 0;

            if (IsBiggerSlipPage(price))
            {
                if (m_Varibles.ExecuteTrades() && Proceed())
                {
                    if (CCurrencyPairs::IsCurrencyPair(symbol))
                    {
                        nr = GetInteroperability().OrderSend(symbol, buy, lotSize, price, CConfig::SlipPage, stopLoss, takeProfit);
                    }
                    else
                    {
                        nr = GetInteroperability().OrderSend(symbol, buy, lotSize, price, CConfig::SlipPage, stopLoss, takeProfit);
                    }
                }
            }
            else
            {
                if (m_Varibles.ExecuteTrades() && Proceed())
                {
                    if (CCurrencyPairs::IsCurrencyPair(symbol))
                    {
                        nr = GetInteroperability().OrderSend(symbol, buy, lotSize, price, CConfig::SlipPage, stopLoss, takeProfit);
                    }
                    else
                    {
                        nr = GetInteroperability().OrderSend(symbol, buy, lotSize, price, CConfig::SlipPage, stopLoss, takeProfit);
                    }
                }
            }

            if (nr >= 0)
            {
                IString msg("Order Send, Pair : ");

                msg += (symbol + ", Price : " + GetInteroperability().DoubleToString(price) + ", Type : " + type + ", Take Profit : " + GetInteroperability().DoubleToString(takeProfit) + ", Stop Loss : " + GetInteroperability().DoubleToString(stopLoss));
                GetLogger().LogTrade(msg);
            }
            else
            {
                IString msg("Error, Order Send, Pair : ");

                msg += (symbol + ", Price : " + GetInteroperability().DoubleToString(price) + ", Type : " + type + ", Take Profit : " + GetInteroperability().DoubleToString(takeProfit) + ", Stop Loss : " + GetInteroperability().DoubleToString(stopLoss));
                GetLogger().LogTrade(msg);

                IString error("Error : ");

                error += (GetInteroperability().IntegerToString(GetInteroperability().GetLastError()));

                GetLogger().LogTrade(error);
            }
            return nr;
        }

		void OrderSelect(int nr)
		{
			m_TicketNr = nr;
			m_Ticket = GetInteroperability().HistoryOrderTicket(nr);
		}

		double OrderOpenPrice()
		{
			return GetInteroperability().OrderOpenPrice(m_Ticket);
		}

		double OrderPrice()
		{
			return GetInteroperability().OrderCurrentPrice(m_Ticket);
		}

		double OrderProfit()
		{
			return GetInteroperability().OrderProfit(m_Ticket);
		}

		double OrderTakeProfit()
		{
			return GetInteroperability().OrderTakeProfit(m_Ticket);
		}

		double OrderStopLoss()
		{
			return GetInteroperability().OrderStopLoss(m_Ticket);
		}
		
        static double LastPeriodClosedSuccessfullTradesAmount(IDateTime &date)
        {
            int total = GetInteroperability().OrdersHistoryTotal();
            int all = 0, positive = 0;
            IDateTime dateOrder;
            unsigned long ticket;
           
            for(int i=0; i < total ;i++)
            {
                ticket = GetInteroperability().HistoryOrderTicket(i);
                if(GetInteroperability().OrderSelect(i, ticket)==true)
                {
                    GetInteroperability().OrderOpenTime(ticket, dateOrder);
                    if(dateOrder > date)
                    {
                        all++;
                        if(GetInteroperability().OrderProfit(ticket) > 0.0)
                        {
                            positive++;
                        }
                    }
                }
            }
            if(all == 0)
            {
                return 0.0;
            }
            return (double)(positive)/(double)(all);
        }

        static bool ProceedWithSearching(CCurrencyPairs &pairs, bool other, int nr, bool buy, double upperLimit, double lowerLimit)
        {
            double takeProfit;
            double stopLoss;
            double price;
            IString symbol;
            
            if(other)
            {
                if(buy)
                {
                    symbol.AssignString(pairs.ReturnOtherSymbolToBuyNr(nr));
                }
                else
                {
                    symbol.AssignString(pairs.ReturnOtherSymbolToSellNr(nr));
                }
            }
            else
            {
                if(buy)
                {
                    symbol.AssignString(pairs.ReturnCurrencyPairToBuyNr(nr));
                }
                else
                {
                    symbol.AssignString(pairs.ReturnCurrencyPairToSellNr(nr));
                }
            }
            
            if(buy)
            {
                price = CValues::BuyPrice(symbol);
                takeProfit = upperLimit;
                stopLoss = lowerLimit;
            }
            else
            {
                price = CValues::SellPrice(symbol);         
                takeProfit = lowerLimit;
                stopLoss = upperLimit;
            }         
            if(buy)
            {
                if((takeProfit - price) * CConfig::TakeProfitPercentForStopLoss * CConfig::TakeProfitPercentForStopLossToIgnore < (price - stopLoss))
                {
                     return false;
                }
            }
            else
            {
                if((price - takeProfit) * CConfig::TakeProfitPercentForStopLoss * CConfig::TakeProfitPercentForStopLossToIgnore < (stopLoss - price))
                {
                     return false;
                }
            }
            return true;
        }
    
        static int SendOrder(CCurrencyPairs &pairs, bool other, int nr, bool buy, double upperLimit, double lowerLimit, bool reverseFromTrend, bool afterTrendReverseTrades, bool automaticReverseTrades, bool methodEReverseTrades, bool reverseTradesMethodG, double LotSize)
        {
            double takeProfit;
            double stopLoss;
            double price;
            IString symbol;
            double spread = -1.0;
            
			m_Varibles.ReadVaribles();
            LotSize = m_Varibles.LotSize();

            if(m_Varibles.ReverseTrades())
            {
                buy = !buy;
            }

			if (automaticReverseTrades)
			{
				buy = !buy;
			}

			if (CConfig::UseTrendFromLogs && reverseFromTrend)
			{
				buy = !buy;
			}

			if (CConfig::UseAfterTrend && afterTrendReverseTrades)
			{
				buy = !buy;
			}

			if (CConfig::UseRules && methodEReverseTrades)
			{
				buy = !buy;
			}

			if (CConfig::ConditionMultivariateTrailStop == MultivariateTrailStop::SURFING_REVERSE_PROFITS && reverseTradesMethodG)
			{
				buy = !buy;
			}

            if(other)
            {
                if(buy)
                {
                    symbol.AssignString(pairs.ReturnOtherSymbolToBuyNr(nr));
                }
                else
                {
                    symbol.AssignString(pairs.ReturnOtherSymbolToSellNr(nr));
                }
            }
            else
            {
                if(buy)
                {
                    symbol.AssignString(pairs.ReturnCurrencyPairToBuyNr(nr));
                }
                else
                {
                    symbol.AssignString(pairs.ReturnCurrencyPairToSellNr(nr));
                }
            }
            
			double modify;
			double middle;

			modify = (upperLimit - lowerLimit) * CConfig::TakeProfitStopLossBiggerSpan;
			middle = (upperLimit + lowerLimit) / 2.0;
			
			if(buy)
            {
			    price = CValues::BuyPrice(symbol);
                takeProfit = middle + modify / 2.0;
                stopLoss = middle - modify / 2.0;
            }
            else
            {
				price = CValues::SellPrice(symbol);
                takeProfit = middle - modify / 2.0;
                stopLoss = middle + modify / 2.0;
            }
            
            GetLogger().LogSpecial(IString("Symbol : ") + symbol + ", Price : " + GetInteroperability().DoubleToString(price) + ", upperLimit : " + GetInteroperability().DoubleToString(upperLimit) + ", LowerLimit : " + GetInteroperability().DoubleToString(lowerLimit));
			GetLogger().LogSpecial(IString("Symbol : ") + symbol + ", ReverseFromTrend : " + GetInteroperability().BoolToString(reverseFromTrend));
			Log(symbol);
            return SendOrder(symbol, price, buy, takeProfit, stopLoss, LotSize);
        }

		static CVaribles* GetVaribles()
		{
			return &m_Varibles;
		}
};
CVaribles CTrading::m_Varibles;
//+------------------------------------------------------------------+
#endif