/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_VARIBLES
#define FILE_VARIBLES

#include "Config.h"
#include "../../Interoperability/String.h"
#include "../../Interoperability/FileWriter.h"
#include "../LibrariesInteroperability.h"
#include "../LibrariesLogger.h"
#include "../../Interoperability/FileReader.h"

//+------------------------------------------------------------------+
class CVaribles {
private:
	static IString m_FileName;
	bool m_ReverseTrades;
	double m_LotSize;
	bool m_ExecuteTrades;
	double m_MultivariateTrailStop;
	double m_MultivariateTrailSpeed;
	double m_MultivariateReverseTrailSpeed;
	double m_MultivariateTrailSpeedHours;
	double m_MultivariateProfitStopMin;
	double m_MultivariateProfitStopMax;
	double m_MultivariateReverseProfitStopMin;
	double m_MultivariateReverseProfitStopMax;
	double m_MultivariateReverseTrades;
	double m_MultivariateBalanceReverseTrades;
	double m_MultivariateSurfingReverseBalanceSpeedMin;
	double m_MultivariateSurfingReverseBalanceSpeedHours;

	static void CreateFile()
	{
		IFileWriter writer;

		writer.Delete(m_FileName);
		writer.Initialize(m_FileName);
		if (CConfig::ReverseTrades)
		{
			writer.Write(IString("ReverseTrades:true"));
		}
		else
		{
			writer.Write(IString("ReverseTrades:false"));
		}
		writer.Write(IString("LotSize:") + GetInteroperability().DoubleToString(CConfig::LotSize));
		if (CConfig::ExecuteTrades)
		{
			writer.Write(IString("ExecuteTrades:true"));
		}
		else
		{
			writer.Write(IString("ExecuteTrades:false"));
		}
		writer.Write(IString("MultivariateTrailStopValue:") + GetInteroperability().DoubleToString(CConfig::MultivariateTrailStopValue));
		writer.Write(IString("MultivariateTrailSpeedValue:") + GetInteroperability().DoubleToString(CConfig::MultivariateTrailSpeedValue));
		writer.Write(IString("MultivariateReverseTrailSpeedValue:") + GetInteroperability().DoubleToString(CConfig::MultivariateReverseTrailSpeedValue));
		writer.Write(IString("MultivariateTrailSpeeddHours:") + GetInteroperability().DoubleToString(CConfig::MultivariateTrailSpeedHours));
		writer.Write(IString("MultivariateProfitStopMin:") + GetInteroperability().DoubleToString(CConfig::MultivariateProfitStopMin));
		writer.Write(IString("MultivariateProfitStopMax:") + GetInteroperability().DoubleToString(CConfig::MultivariateProfitStopMax));
		writer.Write(IString("MultivariateReverseProfitStopMin:") + GetInteroperability().DoubleToString(CConfig::MultivariateReverseProfitStopMin));
		writer.Write(IString("MultivariateReverseProfitStopMax:") + GetInteroperability().DoubleToString(CConfig::MultivariateReverseProfitStopMax));
		writer.Write(IString("MultivariateReverseTradesMin:") + GetInteroperability().DoubleToString(CConfig::MultivariateReverseTradesMin));
		writer.Write(IString("MultivariateBalanceReverseTradesMin:") + GetInteroperability().DoubleToString(CConfig::MultivariateBalanceReverseTradesMin));
		writer.Write(IString("MultivariateSurfingReverseBalanceSpeedHours:") + GetInteroperability().DoubleToString(CConfig::MultivariateSurfingReverseBalanceSpeedHours));
		writer.Write(IString("MultivariateSurfingReverseBalanceSpeedMin:") + GetInteroperability().DoubleToString(CConfig::MultivariateSurfingReverseBalanceSpeedMin));
	}

public:
	CVaribles()
	{
		m_LotSize = CConfig::LotSize;
		m_MultivariateTrailStop = CConfig::MultivariateTrailStopValue;
		m_MultivariateTrailSpeed = CConfig::MultivariateTrailSpeedValue;
		m_MultivariateReverseTrailSpeed = CConfig::MultivariateReverseTrailSpeedValue;
		m_MultivariateTrailSpeedHours = CConfig::MultivariateTrailSpeedHours;
		m_MultivariateProfitStopMin = CConfig::MultivariateProfitStopMin;
		m_MultivariateProfitStopMax = CConfig::MultivariateProfitStopMax;
		m_MultivariateReverseProfitStopMin = CConfig::MultivariateReverseProfitStopMin;
		m_MultivariateReverseProfitStopMax = CConfig::MultivariateReverseProfitStopMax;
		m_MultivariateReverseTrades = CConfig::MultivariateReverseTradesMin;
		m_MultivariateBalanceReverseTrades = CConfig::MultivariateBalanceReverseTradesMin;
		m_MultivariateSurfingReverseBalanceSpeedHours = CConfig::MultivariateSurfingReverseBalanceSpeedHours;
		m_MultivariateSurfingReverseBalanceSpeedMin = CConfig::MultivariateSurfingReverseBalanceSpeedMin;
	}

	static void Init()
	{
		CreateFile();
	}

	void ReadVaribles()
	{
		IFileReader reader;
		IString line;
		IGenericObjectArray<IString> res;
		IString sign(":");

		reader.Initialize(m_FileName);
		line = reader.ReadLine();
		m_ReverseTrades = CConfig::ReverseTrades;
		if (line.Contains(sign))
		{
			line.StringSplit(':', res);
			if (IString::AreEqual((*(res.GetPointerToValue(1))), IString("true")))
			{
				m_ReverseTrades = true;
				GetLogger().LogSpecial(IString("Readed Varibles ReverseTrades = true."));
			}
			else if (IString::AreEqual((*(res.GetPointerToValue(1))), IString("false")))
			{
				m_ReverseTrades = false;
				GetLogger().LogSpecial(IString("Readed Varibles ReverseTrades = false."));
			}
		}
		line = reader.ReadLine();
		m_LotSize = CConfig::LotSize;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_LotSize = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles LotSize = ") + GetInteroperability().DoubleToString(m_LotSize));
		}
		line = reader.ReadLine();
		m_ExecuteTrades = CConfig::ExecuteTrades;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			if (IString::AreEqual((*(res.GetPointerToValue(1))), IString("true")))
			{
				m_ExecuteTrades = true;
				GetLogger().LogSpecial(IString("Readed Varibles ExecuteTrades = true."));
			}
			else if (IString::AreEqual((*(res.GetPointerToValue(1))), IString("false")))
			{
				m_ExecuteTrades = false;
				GetLogger().LogSpecial(IString("Readed Varibles ExecuteTrades = false."));
			}
		}
		line = reader.ReadLine();
		m_MultivariateTrailStop = CConfig::MultivariateTrailStopValue;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_MultivariateTrailStop = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles MultivariateTrailStopValue = ") + GetInteroperability().DoubleToString(m_MultivariateTrailStop));
		}
		line = reader.ReadLine();
		m_MultivariateTrailSpeed = CConfig::MultivariateTrailSpeedValue;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_MultivariateTrailSpeed = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles MultivariateTrailSpeedValue = ") + GetInteroperability().DoubleToString(m_MultivariateTrailSpeed));
		}
		line = reader.ReadLine();
		m_MultivariateReverseTrailSpeed = CConfig::MultivariateReverseTrailSpeedValue;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_MultivariateReverseTrailSpeed = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles MultivariateReverseTrailSpeedValue = ") + GetInteroperability().DoubleToString(m_MultivariateReverseTrailSpeed));
		}
		line = reader.ReadLine();
		m_MultivariateTrailSpeedHours = CConfig::MultivariateTrailSpeedHours;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_MultivariateTrailSpeedHours = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles MultivariateTrailSpeedHours = ") + GetInteroperability().DoubleToString(m_MultivariateTrailSpeedHours));
		}
		line = reader.ReadLine();
		m_MultivariateProfitStopMin = CConfig::MultivariateProfitStopMin;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_MultivariateProfitStopMin = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles MultivariateProfitStopMin = ") + GetInteroperability().DoubleToString(m_MultivariateProfitStopMin));
		}
		line = reader.ReadLine();
		m_MultivariateProfitStopMax = CConfig::MultivariateProfitStopMax;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_MultivariateProfitStopMax = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles MultivariateProfitStopMax = ") + GetInteroperability().DoubleToString(m_MultivariateProfitStopMax));
		}
		line = reader.ReadLine();
		m_MultivariateReverseProfitStopMin = CConfig::MultivariateReverseProfitStopMin;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_MultivariateReverseProfitStopMin = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles MultivariateReverseProfitStopMin = ") + GetInteroperability().DoubleToString(m_MultivariateReverseProfitStopMin));
		}
		line = reader.ReadLine();
		m_MultivariateReverseProfitStopMax = CConfig::MultivariateReverseProfitStopMax;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_MultivariateReverseProfitStopMax = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles MultivariateReverseProfitStopMax = ") + GetInteroperability().DoubleToString(m_MultivariateReverseProfitStopMax));
		}
		line = reader.ReadLine();
		m_MultivariateReverseTrades = CConfig::MultivariateReverseTradesMin;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_MultivariateReverseTrades = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles MultivariateReverseTradesMin = ") + GetInteroperability().DoubleToString(m_MultivariateReverseTrades));
		}
		line = reader.ReadLine();
		m_MultivariateBalanceReverseTrades = CConfig::MultivariateBalanceReverseTradesMin;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_MultivariateBalanceReverseTrades = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles MultivariateBalanceReverseTradesMin = ") + GetInteroperability().DoubleToString(m_MultivariateBalanceReverseTrades));
		}
		line = reader.ReadLine();
		m_MultivariateSurfingReverseBalanceSpeedHours = CConfig::MultivariateSurfingReverseBalanceSpeedHours;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_MultivariateSurfingReverseBalanceSpeedHours = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles MultivariateSurfingReverseBalanceSpeedHours = ") + GetInteroperability().DoubleToString(m_MultivariateSurfingReverseBalanceSpeedHours));
		}
		line = reader.ReadLine();
		m_MultivariateSurfingReverseBalanceSpeedMin = CConfig::MultivariateSurfingReverseBalanceSpeedMin;
		if (line.Contains(sign))
		{
			res.Resize(0);
			line.StringSplit(':', res);
			m_MultivariateSurfingReverseBalanceSpeedMin = GetInteroperability().StringToDouble((*(res.GetPointerToValue(1))));
			GetLogger().LogSpecial(IString("Readed Varibles MultivariateSurfingReverseBalanceSpeedMin = ") + GetInteroperability().DoubleToString(m_MultivariateSurfingReverseBalanceSpeedMin));
		}
	}

	bool ReverseTrades()
	{
		return m_ReverseTrades;
	}

	double LotSize()
	{
		return m_LotSize;
	}

	bool ExecuteTrades()
	{
		return m_ExecuteTrades;
	}

	double MultivariateTrailStop()
	{
		return m_MultivariateTrailStop;
	}

	double MultivariateTrailSpeed()
	{
		return m_MultivariateTrailSpeed;
	}

	double MultivariateReverseTrailSpeed()
	{
		return m_MultivariateReverseTrailSpeed;
	}

	double MultivariateTrailSpeedHours()
	{
		return m_MultivariateTrailSpeedHours;
	}

	double MultivariateProfitStopMin()
	{
		return m_MultivariateProfitStopMin;
	}

	double MultivariateProfitStopMax()
	{
		return m_MultivariateProfitStopMax;
	}

	double MultivariateReverseProfitStopMin()
	{
		return m_MultivariateReverseProfitStopMin;
	}

	double MultivariateReverseProfitStopMax()
	{
		return m_MultivariateReverseProfitStopMax;
	}

	double MultivariateReverseTradesMin()
	{
		return m_MultivariateReverseTrades;
	}

	double MultivariateBalanceReverseTradesMin()
	{
		return m_MultivariateBalanceReverseTrades;
	}

	double MultivariateSurfingReverseBalanceSpeedMin()
	{
		return m_MultivariateSurfingReverseBalanceSpeedMin;
	}

	double MultivariateSurfingReverseBalanceSpeedHours()
	{
		return m_MultivariateSurfingReverseBalanceSpeedHours;
	}
};
IString CVaribles::m_FileName = IString("ConfigurationVaribles.txt");
//+------------------------------------------------------------------+
#endif