/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

#ifndef FILE_SORTABLE_ARRAY
#define FILE_SORTABLE_ARRAY

#ifndef CPP
//+------------------------------------------------------------------+
template <typename T>
class ISortableArray{
	 private:
		 T m_Array[][2];
		 int m_size;
		 
	 public:
		 void Resize(int am)
		 {
			 ::ArrayResize(m_Array, am);
			 m_size = am;
		 }         
		 
		 void SetValue(int nr, const T val)
		 {
			 m_Array[nr][0] = val;
		 }

		 T GetValue(int nr)
		 {
			 return m_Array[nr][0];
		 }
		 
		 void SetPos(int nr, int pos)
		 {
			 m_Array[nr][1] = pos;
		 }

		 int GetPos(int nr)
		 {
			 return (int)m_Array[nr][1];
		 }
		 
		 const int Size()
		 {
			 return m_size;
		 }

		 void Sort()
		 {
			 if (::ArraySize(m_Array) > 0)
			 {
				 ::ArraySort(m_Array);
			 }
		 }

		 int BinarySearch(const T pos)
		 {
			 return ::ArrayBsearch(m_Array, pos);
		 }

		 void RemoveNr(int nr)
		 {
			 for (int i = nr; i < m_size - 2; i++)
			 {
				 m_Array[i][0] = m_Array[i + 1][0];
				 m_Array[i][1] = m_Array[i + 1][1];
			 }
			 Resize(m_size - 1);
		 }
};
//+------------------------------------------------------------------+
#else

#include "../CPP/CalculusRobot/Internal/InternalCppSortableArray.h"

#endif
#endif