﻿/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace CommonLibrary
{
    public class FileLines
    {
        private List<long> Lines { get; set; }
        private FileStream fs;

        private string ReadLine()
        {
            StringBuilder result = new StringBuilder();
            int character;

            while (fs.CanRead && (character = fs.ReadByte()) != '\n' && character != -1 && character != 0)
            {                
                if (character != '\r' && character != '\n')
                {
                    result.Append((char)character);
                }
            }

            return result.ToString();
        }

        public void ReadFile(string name)
        {            
            Lines = new List<long>();
            fs = new FileStream(name, FileMode.Open);
            string line;
            
            ReadLine();
            Lines.Add(0);
            while ((line = ReadLine())!="")
            {
                long l = fs.Position;
                Lines.Add(l);
            }
        }

        public Line GetLineNr(int nr)
        {
            fs.Seek(Lines[nr], SeekOrigin.Begin);
            string line = ReadLine();
            Line l = new Line();
            l.Set(line);
            
            return l;
        }

        public int LinesAmount()
        {
            return Lines.Count - 1;
        }

        public DateTime GetBeginDate()
        {
            Line l = GetLineNr(0);

            return l.Date;
        }

        public DateTime GetEndDate()
        {
            Line l = GetLineNr(Lines.Count - 1);

            return l.Date;
        }
    }
}
