﻿/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using System;
using System.Collections.Generic;

namespace CommonLibrary
{
    class PastProfits
    {
        private double m_Hours;
        private List<double> m_Profits;
        private List<DateTime> m_Dates;

        public PastProfits(double hours)
        {
            m_Hours = hours;
            m_Profits = new List<double>();
            m_Dates = new List<DateTime>();
        }

        private void Move()
        {
            int i;

            for (i = 1; i < m_Profits.Count; i++)
            {
                m_Profits[i - 1] = m_Profits[i];
                m_Dates[i - 1] = m_Dates[i];
            }
            m_Profits.RemoveAt(i - 1);
            m_Dates.RemoveAt(i - 1);
        }

        public void AddProfit(double profit, DateTime date)
        {
            if (m_Profits.Count > 0)
            {
                while ((date - m_Dates[0]).TotalHours > m_Hours)
                {
                    Move();
                    if(m_Profits.Count == 0)
                    {
                        break;
                    }
                }
            }
            m_Profits.Add(profit);
            m_Dates.Add(date);
        }

        public bool IsValid
        {
            get
            {
                if(m_Dates.Count > 1)
                {
                    if((m_Dates[m_Dates.Count - 1] - m_Dates[0]).TotalHours > 0.9 * m_Hours)
                    {
                        return true;
                    }
                }
                return false;
            }
        }

        public double PastProfit
        {
            get
            {
                if (m_Profits.Count > 0)
                {
                    return m_Profits[0];
                }
                else
                {
                    return 0.0;
                }
            }
        }

        public void EmptyPastProfits()
        {
            m_Profits.Clear();
            m_Dates.Clear();
        }
    }
}
