﻿/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using System;
using System.Collections.Generic;

namespace MethodGCalculator
{
    public class Balance
    {
        public double Value { get; set; }
        public DateTime Time { get; set; }
    }
    public class Balances
    {
        public List<Balance> Values { get; private set; }

        private double m_Hours;

        private DateTime m_ResetTime;

        public Balances(double hours)
        {
            Values = new List<Balance>();
            m_Hours = hours;
        }

        public void SetReset(DateTime date)
        {
            m_ResetTime = date;
        }

        public void AddBalance(DateTime time, double balance)
        {
            Balance bal = new Balance();

            bal.Time = time;
            bal.Value = balance;

            Values.Add(bal);
            TrimBegin();
        }

        private void TrimBegin()
        {
            int i;

            for (i = 0; i < Values.Count; i++)
            {
                if (!(DateTimeHelper.DifferenceHours(Values[i].Time, Values[Values.Count - 1].Time) >= m_Hours))
                {
                    break;
                }                
            }
            if (i > 0)
            {
                Values.RemoveRange(0, i - 1);
            }
        }

        public double Speed()
        {
            return (Values[Values.Count - 1].Value - Values[0].Value) / m_Hours;
        }

        public bool IsSpeedValid()
        {
            if(Values.Count > 0)
            {
                if(DateTimeHelper.DifferenceHours(Values[0].Time, Values[Values.Count - 1].Time) >= m_Hours && DateTimeHelper.DifferenceHours(m_ResetTime, Values[Values.Count - 1].Time) >= m_Hours)
                {
                    return true;
                }
            }
            return false;
        }

        public double FinalBalance()
        {
            if(Values.Count > 0)
            {
                return Values[Values.Count - 1].Value;
            }
            else
            {
                return 0.0;
            }
        }
    }
}