﻿/*Copyright 2024 Inconsoft Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using CommonLibrary;
using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.Linq;

namespace MethodGCalculator
{
    class Program
    {
        private static ProfitsModifier pm = new ProfitsModifier();

        static void CalculateWindows(FileLooks looks, int windowSpan, int amount, double diff, bool nmts, int sumsAmount)
        {
            List<int> list = new List<int>();
            DateTime date = looks.Looks[looks.Looks.Count - 1].Date;
            LookSimulator ls = new LookSimulator();

            list.Add(looks.Looks.Count - 1);
            for (int i = looks.Looks.Count - 1; i >= 0; i--)
            {
                if ((date - looks.Looks[i].Date).TotalHours >= windowSpan)
                {
                    list.Add(i);
                    date = looks.Looks[i].Date;
                }
            }

            List<Parts> parts = new List<Parts>();

            for (int i = list.Count - 1; i > 0; i--)
            {
                List<Results> results = new List<Results>();
                int posAmount, negAmount;
                double profit;
                Parts part = new Parts();

                ls.SetLooks(looks.GetSubSetOfLooks(list[i], list[i - 1]));
                for (int j = 0; j < amount; j++)
                {
                    profit = ls.CalculateProfit(j * diff, out posAmount, out negAmount, nmts);

                    Results res = new Results();

                    res.Diff = j * diff;
                    res.Profit = profit;
                    res.PosAmount = posAmount;
                    res.NegAmount = negAmount;
                    results.Add(res);
                }
                List<Results> sums = new List<Results>();

                int sumsDiff = results.Count / sumsAmount;
                int sumam = 0;
                Results resSum = new Results();

                for (int j = 0; j < results.Count; j++)
                {
                    if (j == 0 || j % sumsDiff != 0)
                    {
                        resSum.Diff += results[j].Diff;
                        resSum.Profit += results[j].Profit;
                        resSum.PosAmount += results[j].PosAmount;
                        resSum.NegAmount += results[j].NegAmount;
                        sumam++;
                    }
                    else
                    {
                        resSum.Diff /= sumam;
                        resSum.Profit /= sumam;
                        resSum.NegAmount /= sumam;
                        resSum.PosAmount /= sumam;
                        sumam = 0;
                        sums.Add(resSum);
                        part.Profits.Add(resSum.Profit);
                        resSum = new Results();
                    }
                }
                parts.Add(part);
                Console.WriteLine();
                foreach (Results rs in sums)
                {
                    Console.WriteLine("Profit : {0}, MultivariateTrailStopValue : {1}, PosAmount : {2}, NegAmount : {3}.", rs.Profit, rs.Diff, rs.PosAmount, rs.NegAmount);
                }
                Console.WriteLine();
            }
            DisplayParts(parts);
        }

        static void DisplayParts(List<Parts> parts)
        {
            Parts mins = new Parts(parts[0]);
            Parts maxs = new Parts(parts[0]);

            foreach (Parts p in parts)
            {
                for (int i = 0; i < p.Profits.Count; i++)
                {
                    if (p.Profits[i] < mins.Profits[i])
                    {
                        mins.Profits[i] = p.Profits[i];
                    }
                    if (p.Profits[i] > maxs.Profits[i])
                    {
                        maxs.Profits[i] = p.Profits[i];
                    }
                }
            }
            foreach (Parts p in parts)
            {
                for (int i = 0; i < p.Profits.Count; i++)
                {
                    p.Profits[i] = (p.Profits[i] - mins.Profits[i]) / (maxs.Profits[i] - mins.Profits[i]);
                }
            }
            foreach (Parts p in parts)
            {
                for (int i = 0; i < p.Profits.Count; i++)
                {
                    p.Average += p.Profits[i];
                }
                p.Average /= p.Profits.Count;
            }
            foreach (Parts p in parts)
            {
                Console.Write("{0} ", p.Average);
            }
        }

        static void Display()
        {
            Console.WriteLine("MethodGCalculator -mts ProfitPastValues.txt 300 30 4 [-nmst] [-rp] [-w 24]");
            Console.WriteLine("MethodGCalculator -speed ProfitPastValues.txt [-rp|-prp] 30 2.0");
            Console.WriteLine("MethodGCalculator -balance ProfitPastValues.txt");
            Console.WriteLine("MethodGCalculator -profit ProfitPastValues.txt 300");
            Console.WriteLine("MethodGCalculator -reverseprofits ProfitPastValues.txt 300");
            Console.WriteLine("MethodGCalculator -traderreport_old DetailedStatement.htm Reverses.txt 12 72 0.5 100 20 20 20");
            Console.WriteLine("MethodGCalculator -traderreport DetailedStatement.htm Reverses.txt 36 108 0.5 100");
            Console.WriteLine("MethodGCalculator -closedtrades closed_trades.txt 36 108 0.5 100");
            Console.ReadLine();
        }

        static void Mts(string[] args)
        {
            if (args.Count() < 4)
            {
                Display();
                return;
            }

            string filename;
            int amount;
            int sumsAmount;
            double min, max, diff;
            double profit;
            bool nmts = false;
            int hours = 0;
            int multiAm;

            filename = args[0];
            amount = int.Parse(args[1]);
            sumsAmount = int.Parse(args[2]);
            multiAm = int.Parse(args[3]);

            FileLines fl = new FileLines();
            FileLooks flooks = new FileLooks();
            LookSimulator ls = new LookSimulator();

            fl.ReadFile(filename);
            flooks.FillLooks(fl);
            ls.SetLooks(flooks);

            ls.NormalizeProfits();

            if ((args.Count() == 5 || args.Count() == 7) && args[4] == "-rp" || (args.Count() == 6 || args.Count() == 8) && args[5] == "-rp")
            {
                ls.ReverseProfits();
            }

            if ((args.Count() == 5 || args.Count() == 7) && args[4] == "-nmts" || (args.Count() == 6 || args.Count() == 8) && args[4] == "-nmts")
            {
                nmts = true;
            }

            if (args[args.Count() - 2] == "-w")
            {
                hours = int.Parse(args[args.Count() - 1]);
            }

            ls.MinMxProfits(out min, out max);
            diff = (max - min) / (multiAm * amount);

            if (hours == 0)
            {
                List<Results> results = new List<Results>();
                int posAmount, negAmount;

                for (int i = 0; i < amount; i++)
                {
                    profit = ls.CalculateProfit(i * diff, out posAmount, out negAmount, nmts);
                    Console.Write(i + " ");

                    Results res = new Results();

                    res.Diff = i * diff;
                    res.Profit = profit;
                    res.PosAmount = posAmount;
                    res.NegAmount = negAmount;
                    results.Add(res);
                }

                List<Results> sums = new List<Results>();

                int sumsDiff = results.Count / sumsAmount;
                int sumam = 0;
                Results resSum = new Results();

                for (int i = 0; i < results.Count; i++)
                {
                    if (i == 0 || i % sumsDiff != 0)
                    {
                        resSum.Diff += results[i].Diff;
                        resSum.Profit += results[i].Profit;
                        resSum.PosAmount += results[i].PosAmount;
                        resSum.NegAmount += results[i].NegAmount;
                        sumam++;
                    }
                    else
                    {
                        resSum.Diff /= sumam;
                        resSum.Profit /= sumam;
                        resSum.NegAmount /= sumam;
                        resSum.PosAmount /= sumam;
                        sumam = 0;
                        sums.Add(resSum);
                        resSum = new Results();
                    }
                }
                results.Sort();
                foreach (Results rs in results)
                {
                    Console.WriteLine("Profit : {0}, MultivariateTrailStopValue : {1}, PosAmount : {2}, NegAmount : {3}.", rs.Profit, rs.Diff, rs.PosAmount, rs.NegAmount);
                }
                Console.WriteLine();
                foreach (Results rs in sums)
                {
                    Console.WriteLine("Profit : {0}, MultivariateTrailStopValue : {1}, PosAmount : {2}, NegAmount : {3}.", rs.Profit, rs.Diff, rs.PosAmount, rs.NegAmount);
                }
                Console.ReadLine();
            }
            else
            {
                CalculateWindows(flooks, hours, amount, diff, nmts, sumsAmount);
                Console.ReadLine();
            }
        }

        static void PerformRunner(List<Profit> profits, double minHours, double maxHours, double diff, int amSpeeds)
        {
            List<MostCommonSpeedHours> lmcsh = new List<MostCommonSpeedHours>();
            List<SpeedInfo> speedsInfo = new List<SpeedInfo>();
            int amPos;
            int pos = 0;
            double min = 0.0;

            for (double hours = minHours; hours <= maxHours; hours += diff)
            {
                ProfitsRunner runner = new ProfitsRunner(hours);

                runner.Profits = profits;
                speedsInfo.AddRange(runner.Run(amSpeeds));
                Console.Write("[[[[{0}]]]] ,", hours);
            }
            speedsInfo.Sort();
            amPos = speedsInfo.Count / 100;
            foreach (Profit prof in profits)
            {
                if (min > prof.GainLoss)
                {
                    min = prof.GainLoss;
                }
            }
            Console.WriteLine();
            Console.WriteLine();
            for (int i = 0; i < speedsInfo.Count; i++)
            {
                MostCommonSpeedHours mcsh = new MostCommonSpeedHours();

                mcsh.Info = speedsInfo[i];
                foreach (SpeedInfo other in speedsInfo)
                {
                    AddCorrelations(mcsh, other);
                }
                mcsh.MultiplyCorrelations();
                if (mcsh.Info.Profit != 0.0)
                {
                    lmcsh.Add(mcsh);
                }
                if (i % amPos == 0)
                {
                    Console.Write("{0}, ", pos);
                    pos++;
                }
            }
            Console.WriteLine();
            lmcsh.Sort();
            foreach (MostCommonSpeedHours mh in lmcsh)
            {
                Console.WriteLine("Profit:{0}, Hours:{1}, Speed:{2}.", mh.Info.Profit, mh.Info.Hours, mh.Info.Speed);
            }
            Console.WriteLine();
            Console.WriteLine("Most Common : Profit:{0}, Hours:{1}, Speed:{2}.", lmcsh[lmcsh.Count - 1].Info.Profit, lmcsh[lmcsh.Count - 1].Info.Hours, lmcsh[lmcsh.Count - 1].Info.Speed);
            Console.WriteLine();
            Console.WriteLine();
            CalculateRank(lmcsh);
            Console.ReadLine();
        }

        static void CalculateRank(List<MostCommonSpeedHours> lmcsh)
        {
            List<Rank> ranks = new List<Rank>();
            Dictionary<double, HoursInfo> dict = new Dictionary<double, HoursInfo>();

            for (int i = 0; i < lmcsh.Count; i++)
            {
                Rank rank = new Rank();

                rank.Mcsh = lmcsh[i];
                rank.PositionInMostCommon = i / (lmcsh.Count - 1.0);

                if (dict.ContainsKey(lmcsh[i].Info.Hours))
                {
                    if (lmcsh[i].Info.Profit > 0.0)
                    {
                        dict[lmcsh[i].Info.Hours].PosOccurence++;
                        dict[lmcsh[i].Info.Hours].PosProfits += lmcsh[i].Info.Profit;
                    }
                    else
                    {
                        dict[lmcsh[i].Info.Hours].NegOccurence++;
                        dict[lmcsh[i].Info.Hours].NegProfits += lmcsh[i].Info.Profit;
                    }
                }
                else
                {
                    dict.Add(lmcsh[i].Info.Hours, new HoursInfo());
                    if (lmcsh[i].Info.Profit > 0.0)
                    {
                        dict[lmcsh[i].Info.Hours].PosOccurence = 1;
                        dict[lmcsh[i].Info.Hours].PosProfits = lmcsh[i].Info.Profit;
                    }
                    else
                    {
                        dict[lmcsh[i].Info.Hours].NegOccurence = 1;
                        dict[lmcsh[i].Info.Hours].NegProfits = lmcsh[i].Info.Profit;
                    }
                }
                rank.Hours = dict[lmcsh[i].Info.Hours];
                ranks.Add(rank);
            }
            foreach (Rank rank in ranks)
            {
                rank.HoursRank = (rank.Hours.PosProfits + rank.Hours.NegProfits);
                rank.HoursRank *= rank.PositionInMostCommon;
            }
            ranks.Sort();
            foreach (Rank rank in ranks)
            {
                Console.WriteLine("Profit:{0}, Hours:{1}, Speed:{2}, HoursPercentagePosOccurence:{3}.", rank.Mcsh.Info.Profit, rank.Mcsh.Info.Hours, rank.Mcsh.Info.Speed, (rank.Hours.PosOccurence) / (rank.Hours.PosOccurence + rank.Hours.NegOccurence + 0.0) * 100.0);
            }
        }

        static void AddCorrelations(MostCommonSpeedHours mcsh, SpeedInfo si)
        {
            mcsh.CorrelationsSpeeds += (Math.Abs(mcsh.Info.Profit - si.Profit) / (1.0 + Math.Abs((mcsh.Info.Speed - si.Speed) != 0.0 ? (mcsh.Info.Speed - si.Speed) : 1.0)));
            mcsh.CorrelationsHours += (Math.Abs(mcsh.Info.Profit - si.Profit) / (1.0 + Math.Abs((mcsh.Info.Hours - si.Hours) != 0.0 ? (mcsh.Info.Hours - si.Hours) : 1.0)));
        }

        static void PerformRunner(List<Profit> profits, double minHours, double maxHours, double diff, int amSpeeds, int amAverages, int amRangesSpeeds, int amRangesHours)
        {
            List<SpeedInfo> speedsInfo = new List<SpeedInfo>();
            List<SpeedAverageInfo> speedsAverage = new List<SpeedAverageInfo>();
            SpeedAverageInfo sai = new SpeedAverageInfo();
            int j = 1;

            for (double hours = minHours; hours <= maxHours; hours += diff)
            {
                ProfitsRunner runner = new ProfitsRunner(hours);

                runner.Profits = profits;
                speedsInfo.AddRange(runner.Run(amSpeeds));
                Console.Write("{0} ,", hours);
            }
            speedsInfo.Sort();

            for (int i = speedsInfo.Count - 1; i >= 0; i--)
            {
                sai.Hours += speedsInfo[i].Hours;
                sai.Profit += speedsInfo[i].Profit;
                sai.Speed += speedsInfo[i].Speed;

                if (j % amAverages == 0)
                {
                    sai.Hours /= amAverages;
                    sai.Profit /= amAverages;
                    sai.Speed /= amAverages;

                    speedsAverage.Add(sai);
                    sai = new SpeedAverageInfo();
                }
                j++;
            }

            foreach (SpeedInfo si in speedsInfo)
            {
                Console.WriteLine("Profit : {0}, Hours : {1}, Speed : {2}.", si.Profit, si.Hours, si.Speed);
            }
            Console.WriteLine();
            for (int i = speedsAverage.Count - 1; i >= 0; i--)
            {
                Console.WriteLine("Averages, Profit : {0}, Hours : {1}, Speed : {2}.", speedsAverage[i].Profit, speedsAverage[i].Hours, speedsAverage[i].Speed);
            }
            Console.WriteLine();
            DisplayStatisticsOfRegions(speedsInfo, amRangesSpeeds, amRangesHours);
            Console.ReadLine();
        }

        static void DisplayStatisticsOfRegions(List<SpeedInfo> si, int amRangesSpeeds, int amRangesHours)
        {
            List<MostCommonSpeedHours> mcsh = new List<MostCommonSpeedHours>();
            List<RangesSpeeds> ranges = new List<RangesSpeeds>();
            double minSpeed = 0.0;
            double maxSpeed = 0.0;
            double minHours = 0.0;
            double maxHours = 0.0;
            double pos, neg;
            double diffSpeed;
            double diffHours;
            int rangeNr = 0;
            List<SpeedInfo> resSI = new List<SpeedInfo>();

            foreach (SpeedInfo s in si)
            {
                if (minSpeed > s.Speed)
                {
                    minSpeed = s.Speed;
                }
                if (maxSpeed < s.Speed)
                {
                    maxSpeed = s.Speed;
                }
            }
            diffSpeed = (maxSpeed - minSpeed) / amRangesSpeeds;
            for (double min = minSpeed; min < maxSpeed; min += diffSpeed)
            {
                pos = 0.0;
                neg = 0.0;

                foreach (SpeedInfo s in si)
                {
                    if (s.Speed >= min && s.Speed <= min + diffSpeed)
                    {
                        if (s.Profit > 0.0)
                        {
                            pos += 1.0;
                        }
                        else if (s.Profit < 0.0)
                        {
                            neg += 1.0;
                        }
                    }
                }
                RangesSpeeds rs = new RangesSpeeds();

                rs.MinSpeed = min;
                rs.MaxSpeed = min + diffSpeed;
                rs.NegativePercentage = neg / (pos + neg) * 100.0;
                rs.PositivePercentage = pos / (pos + neg) * 100.0;

                ranges.Add(rs);
            }
            foreach (RangesSpeeds rs in ranges)
            {
                Console.WriteLine("Speeds:{0} - {1}, PosPercent:{2}, NegPercent:{3}.", rs.MinSpeed, rs.MaxSpeed, rs.PositivePercentage, rs.NegativePercentage);
            }
            Console.WriteLine();
            for (int i = 0; i < ranges.Count; i++)
            {
                if (ranges[rangeNr].PositivePercentage < ranges[i].PositivePercentage)
                {
                    rangeNr = i;
                }
            }
            Console.WriteLine("Speeds: {0} - {1}.", ranges[rangeNr].MinSpeed, ranges[rangeNr].MaxSpeed);
            Console.WriteLine();
            foreach (SpeedInfo s in si)
            {
                if (minHours > s.Hours)
                {
                    minHours = s.Hours;
                }
                if (maxHours < s.Hours)
                {
                    maxHours = s.Hours;
                }
            }
            diffHours = (maxHours - minHours) / amRangesHours;
            for (double min = minHours; min < maxHours; min += diffHours)
            {
                resSI.Clear();
                pos = 0.0;
                neg = 0.0;

                foreach (SpeedInfo s in si)
                {
                    if (s.Hours >= min && s.Hours <= min + diffHours && s.Speed >= ranges[rangeNr].MinSpeed && s.Speed <= ranges[rangeNr].MaxSpeed)
                    {
                        if (s.Profit > 0.0)
                        {
                            resSI.Add(s);
                            pos += 1.0;
                        }
                        else if (s.Profit < 0.0)
                        {
                            neg += 1.0;
                        }
                    }
                }
                Console.WriteLine("Hours:{0} - {1}, PosPercent:{2}, NegPercent:{3}.", min, min + diffSpeed, pos / (pos + neg) * 100.0, neg / (pos + neg) * 100.0);
                Console.WriteLine();
                foreach (SpeedInfo rsi in resSI)
                {
                    Console.WriteLine("Profit : {0}, Hours : {1}, Speed : {2}.", rsi.Profit, rsi.Hours, rsi.Speed);
                }
                mcsh.Add(MostCommouSpeedHour(resSI, amRangesSpeeds, amRangesHours));
            }
            Console.WriteLine();
            for (int i = 0; i < mcsh.Count; i++)
            {
                if (mcsh[i] != null)
                {
                    Console.WriteLine("Most Common Hours and Speed : Profit:{0}, Hours:{1}, Speed:{2}.", mcsh[i].Info.Profit, mcsh[i].Info.Hours, mcsh[i].Info.Speed);
                }
            }
            Console.WriteLine();
        }

        static MostCommonSpeedHours MostCommouSpeedHour(List<SpeedInfo> lsi, int amRangeSpeeds, int amRangeHours)
        {
            MostCommonSpeedHours sh = new MostCommonSpeedHours();
            double minSpeed = 0.0;
            double maxSpeed = 0.0;
            double minHours = 0.0;
            double maxHours = 0.0;
            double diffSpeed = 0.0;
            double diffHours = 0.0;
            int siNr = 0;

            foreach (SpeedInfo si in lsi)
            {
                if (minSpeed > si.Speed)
                {
                    minSpeed = si.Speed;
                }
                else if (maxSpeed < si.Speed)
                {
                    maxSpeed = si.Speed;
                }
            }
            diffSpeed = (maxSpeed - minSpeed) / amRangeSpeeds;
            foreach (SpeedInfo si in lsi)
            {
                foreach (SpeedInfo sig in lsi)
                {
                    if (si.Speed >= sig.Speed - diffSpeed && si.Speed <= sig.Speed + diffSpeed)
                    {
                        si.CloseSpeedsCount++;
                    }
                }
            }
            foreach (SpeedInfo si in lsi)
            {
                if (minHours > si.Hours)
                {
                    minHours = si.Hours;
                }
                else if (maxHours < si.Hours)
                {
                    maxHours = si.Hours;
                }
            }
            diffHours = (maxHours - minHours) / amRangeHours;
            foreach (SpeedInfo si in lsi)
            {
                foreach (SpeedInfo sig in lsi)
                {
                    if (si.Hours >= sig.Hours - diffHours && si.Hours <= sig.Hours + diffHours)
                    {
                        si.CloseHoursCount++;
                    }
                }
            }
            for (int i = 0; i < lsi.Count; i++)
            {
                if (lsi[siNr].CloseHoursCount * lsi[siNr].CloseSpeedsCount < lsi[i].CloseHoursCount * lsi[i].CloseSpeedsCount)
                {
                    siNr = i;
                }
            }
            if (siNr > 0)
            {
                sh.Info = lsi[siNr];

                return sh;
            }
            return null;
        }

        static void TraderReport_old(string name, string nameReverses, double minHours, double maxHours, double diff, int amSpeeds, int amAverages, int amRangesSpeeds, int amRangesHours)
        {
            pm.Prepare(name, nameReverses);
            pm.Save("results.txt");

            PerformRunner(pm.Profits, minHours, maxHours, diff, amSpeeds, amAverages, amRangesSpeeds, amRangesHours);
        }

        static void TraderReport(string name, string nameReverses, double minHours, double maxHours, double diff, int amSpeeds)
        {
            pm.Prepare(name, nameReverses);
            pm.Save("results.txt");
           
            PerformRunner(pm.Profits, minHours, maxHours, diff, amSpeeds);
        }

        static void ClosedTrades(string name, double minHours, double maxHours, double diff, int amSpeeds)
        {
            pm.Prepare(name);
            pm.Save("results.txt");
            
            PerformRunner(pm.Profits, minHours, maxHours, diff, amSpeeds);
        }

        static void Speeds(LookSimulator ls, int amount, double hours, out List<Results> results)
        {
            double min, max;

            ls.MinMaxSpeed(hours, out min, out max);

            min = min / 2.0;
            max = max / 5.0;

            results = new List<Results>();
            double profit;
            double diff = (max - min) / (amount);
            int closedAmount;

            for (int i = 0; i < amount; i++)
            {
                profit = ls.CalculateProfitWithSpeed(min + i * diff, hours, out closedAmount);
                Console.Write(i + " ");

                Results res = new Results();

                res.Diff = min + i * diff;
                res.Profit = profit;
                res.ClosedAmount = closedAmount;
                results.Add(res);
            }

            results.Sort();
            foreach (Results rs in results)
            {
                Console.WriteLine("Profit : {0}, Speed : {1}, Closed Amount : {2}.", rs.Profit, rs.Diff, rs.ClosedAmount);
            }
            Console.WriteLine();
            Console.WriteLine("Min Speed : {0}, Max Speed : {1}.", min, max);

        }

        static void Profits(LookSimulator ls, int amount, out List<Results> results)
        {
            results = new List<Results>();
            double profit;
            double max;
            double min;
            int closedAmountMin, closedAmountMax;
            int removedMinAm, removedMaxAm;

            ls.MinMxProfits(out min, out max);
            max /= 15.0;

            double diff = max / amount;


            for (int i = 0; i < amount; i++)
            {
                Console.Write(i + " ");
                profit = ls.CalculateProfitWithProfit(min, 0 + i * diff, out closedAmountMin, out closedAmountMax, out removedMinAm, out removedMaxAm);

                Results res = new Results();

                res.ProfitMin = min;
                res.ProfitMax = 0 + i * diff;
                res.Profit = profit;
                res.ClosedAmountMin = closedAmountMin;
                res.ClosedAmountMax = closedAmountMax;
                res.RemovedMinAm = removedMinAm;
                res.RemovedMaxAm = removedMaxAm;
                results.Add(res);
            }

            results.Sort();
            foreach (Results rs in results)
            {
                if (rs.ProfitMin < rs.ProfitMax)
                {
                    Console.WriteLine("Profit : {0}, ProfitMin : {1}, ProfitMax : {2}, Closed Amount Min : {3}, Closed Amount Max : {4}, Avg Min Removed : {5}, Avg Max Removed : {6}.", rs.Profit, rs.ProfitMin, rs.ProfitMax, rs.ClosedAmountMin, rs.ClosedAmountMax, rs.ClosedAmountMin != 0 ? rs.RemovedMinAm / (double)rs.ClosedAmountMin : 0.0, rs.ClosedAmountMax != 0 ? rs.RemovedMaxAm / (double)rs.ClosedAmountMax : 0.0);
                }
            }
            Console.WriteLine();
            Console.WriteLine("Min Profit : {0}, Max Profit : {1}.", min, max);
        }

        static void ReverseProfits(LookSimulator ls, int amount, out List<Results> results)
        {
            results = new List<Results>();
            double profit;
            double min, max;
            int closedAmountMin;
            int removedMinAm;

            ls.MinMxProfits(out min, out max);

            double diff = -min / amount;


            for (int i = 0; i < amount; i++)
            {
                Console.Write(i + " ");
                profit = ls.CalculateProfitWithReversingProfit(min + i * diff, out closedAmountMin, out removedMinAm);

                Results res = new Results();

                res.ProfitMin = min + i * diff;
                res.Profit = profit;
                res.ClosedAmountMin = closedAmountMin;
                res.RemovedMinAm = removedMinAm;
                results.Add(res);
            }

            results.Sort();
            foreach (Results rs in results)
            {
                if (rs.ProfitMin < rs.ProfitMax)
                {
                    Console.WriteLine("Profit : {0}, ProfitMin : {1}, Closed Amount Min : {2}, Avg Min Removed : {3}.", rs.Profit, rs.ProfitMin, rs.ClosedAmountMin, rs.ClosedAmountMin != 0 ? rs.RemovedMinAm / (double)rs.ClosedAmountMin : 0.0);
                }
            }
            Console.WriteLine();
            Console.WriteLine("Min Profit : {0}.", min);
        }

        static List<Res> DetermineBestSpeeds(List<Results> p, List<Results> rp)
        {
            Results res;
            List<Res> results = new List<Res>();
            Res re;

            foreach (Results r in p)
            {
                res = Res.DetermineBestMatch(r, rp);
                if (res != null)
                {
                    re = new Res();
                    re.Profit = r.Profit;
                    re.ReverseProfit = res.Profit;
                    re.ClosedAmount = r.ClosedAmount;
                    re.ReverseClosedAmount = res.ClosedAmount;
                    re.Speed = r.Diff;
                    re.ReverseSpeed = res.Diff;
                    results.Add(re);
                }
            }
            return results;
        }

        static void Speed(string[] args)
        {
            bool rp = false;
            bool prp = false;

            if (args.Count() < 3)
            {
                Display();
                return;
            }
            else if (args.Count() < 4 && args[1] != "-rp" && args[1] != "-prp")
            {

            }
            else if (args.Count() == 4 && args[1] == "-rp")
            {
                List<string> tmpL = (args.ToList());

                rp = true;
                tmpL.RemoveAt(1);
                args = tmpL.ToArray();
            }
            else if (args.Count() == 4 && args[1] == "-prp")
            {
                List<string> tmpL = (args.ToList());

                prp = true;
                tmpL.RemoveAt(1);
                args = tmpL.ToArray();
            }

            string filename;
            int amount;
            double hours;

            filename = args[0];
            amount = int.Parse(args[1]);
            hours = double.Parse(args[2]);

            FileLines fl = new FileLines();
            FileLooks flooks = new FileLooks();
            LookSimulator ls = new LookSimulator();

            fl.ReadFile(filename);
            flooks.FillLooks(fl);
            ls.SetLooks(flooks);

            ls.NormalizeProfits();

            if (rp)
            {
                ls.ReverseProfits();
            }
            else if (prp)
            {
                List<Results> p;
                List<Results> rpl;

                Speeds(ls, amount, hours, out p);
                ls.ClearRemovedProfits();
                ls.ReverseProfits();
                Speeds(ls, amount, hours, out rpl);
                ls.Clear();

                List<Res> re = DetermineBestSpeeds(p, rpl);

                re.Sort();
                foreach (Res r in re)
                {
                    Console.WriteLine("Overall Profit : {0}, Profit : {1}, ReverseProfit : {2}, Speed : {3}, ReverseSpeed : {4}, ClosedAmount : {5}, ReverseClosedAmount : {6}.", r.Profit + r.ReverseProfit, r.Profit, r.ReverseProfit, r.Speed, r.ReverseSpeed, r.ClosedAmount, r.ReverseClosedAmount);
                }
                Console.ReadLine();
                return;
            }
            List<Results> res;
            Speeds(ls, amount, hours, out res);
            ls.Clear();
            Console.ReadLine();
        }

        static void Balance(string[] args)
        {
            if (args.Count() < 1)
            {
                Display();
                return;
            }
            FileLines fl = new FileLines();
            FileLooks flooks = new FileLooks();
            LookSimulator ls = new LookSimulator();

            fl.ReadFile(args[0]);
            flooks.FillLooks(fl);
            ls.SetLooks(flooks);

            ls.NormalizeProfits();

            double differenceSum;
            double balance = ls.CalculateBalance(out differenceSum);

            Console.WriteLine("Balance : {0}, DifferenceSum : {1}.", balance, differenceSum);
            Console.ReadLine();

        }

        static void ReverseProfits(string[] args)
        {
            if (args.Count() < 2)
            {
                Display();
                return;
            }
            FileLines fl = new FileLines();
            FileLooks flooks = new FileLooks();
            LookSimulator ls = new LookSimulator();

            fl.ReadFile(args[0]);
            flooks.FillLooks(fl);
            ls.SetLooks(flooks);

            int amount;

            amount = int.Parse(args[1]);

            ls.NormalizeProfits();

            List<Results> p;

            ReverseProfits(ls, amount, out p);
            ls.Clear();

            Console.ReadLine();
            return;
        }

        static void Profit(string[] args)
        {
            if (args.Count() < 2)
            {
                Display();
                return;
            }
            FileLines fl = new FileLines();
            FileLooks flooks = new FileLooks();
            LookSimulator ls = new LookSimulator();

            fl.ReadFile(args[0]);
            flooks.FillLooks(fl);
            ls.SetLooks(flooks);

            int amount;

            amount = int.Parse(args[1]);

            ls.NormalizeProfits();

            List<Results> p;
            List<Results> rpl;

            Profits(ls, amount, out p);
            ls.ClearRemovedProfits();
            ls.ReverseProfits();
            Profits(ls, amount, out rpl);
            ls.Clear();

            Console.ReadLine();
            return;
        }

        static void Main(string[] args)
        {
            if (args.Count() < 1)
            {
                Display();
                return;
            }
            List<string> argsN = new List<string>();

            for (int i = 1; i < args.Count(); i++)
            {
                argsN.Add(args[i]);
            }

            if (args[0] == "-mts")
            {
                Mts(argsN.ToArray());
                return;
            }
            else if (args[0] == "-speed")
            {
                Speed(argsN.ToArray());
                return;
            }
            else if (args[0] == "-balance")
            {
                Balance(argsN.ToArray());
                return;
            }
            else if (args[0] == "-profit")
            {
                Profit(argsN.ToArray());
                return;
            }
            else if (args[0] == "-reverseprofits")
            {
                ReverseProfits(argsN.ToArray());
                return;
            }
            else if (args[0] == "-traderreport_old")
            {
                TraderReport_old(args[1], args[2], double.Parse(args[3]), double.Parse(args[4]), double.Parse(args[5]), int.Parse(args[6]), int.Parse(args[7]), int.Parse(args[8]), int.Parse(args[9]));
                return;
            }
            else if (args[0] == "-traderreport")
            {
                TraderReport(args[1], args[2], double.Parse(args[3]), double.Parse(args[4]), double.Parse(args[5]), int.Parse(args[6]));
                return;
            }
            else if (args[0] == "-closedtrades")
            {
                ClosedTrades(args[1], double.Parse(args[2]), double.Parse(args[3]), double.Parse(args[4]), int.Parse(args[5]));
                return;
            }
            else
            {
                Display();
                return;
            }
        }
    }
}
